@@ -134,6 +134,8 @@ static ILBuiltinType const ILUnmanagedPtr =
 	{&ILSystem##type, 0, 0},
 #define	IL_EXPLICIT_OPERATOR(type)	\
 	{&ILSystem##type, 1, 0},
+#define	IL_UNSAFE_OPERATOR_REVERSE	\
+	{&ILUnmanagedPtr, 1, 1},
 #define	IL_UNSAFE_OPERATOR(type)	\
 	{&ILSystem##type, 1, 1},
 #define	IL_END_CONVERT_TABLE	\
@@ -431,6 +433,32 @@ IL_END_OPERATOR_TABLE
 
 /*
 
+int operator>>>(int x, int count);
+uint operator>>>(uint x, int count);
+long operator>>>(long x, int count);
+ulong operator>>>(ulong x, int count);
+E operator>>>(E x, int count)
+U operator>>>(E x, int count)
+
+*/
+
+IL_BEGIN_OPERATOR_TABLE(UShr)
+	IL_BINARY_OPERATOR(Int32, Int32, Int32)
+	IL_BINARY_OPERATOR(UInt32, UInt32, Int32)
+	IL_BINARY_OPERATOR(Int64, Int64, Int32)
+	IL_BINARY_OPERATOR(UInt64, UInt64, Int32)
+	{&ILEnumByte, &ILEnumByte, &ILSystemInt32},
+	{&ILEnumSByte, &ILEnumSByte, &ILSystemInt32},
+	{&ILEnumInt16, &ILEnumInt16, &ILSystemInt32},
+	{&ILEnumUInt16, &ILEnumUInt16, &ILSystemInt32},
+	{&ILEnumInt32, &ILEnumInt32, &ILSystemInt32},
+	{&ILEnumUInt32, &ILEnumUInt32, &ILSystemInt32},
+	{&ILEnumInt64, &ILEnumInt64, &ILSystemInt32},
+	{&ILEnumUInt64, &ILEnumUInt64, &ILSystemInt32},
+IL_END_OPERATOR_TABLE
+
+/*
+
 bool operator==(int x, int y);
 bool operator==(uint x, uint y);
 bool operator==(long x, long y);
@@ -869,6 +897,7 @@ explicit operator ushort(int x);
 explicit operator uint(int x);
 explicit operator ulong(int x);
 explicit operator char(int x);
+explicit operator void *(int x);
 
 */
 
@@ -886,6 +915,7 @@ IL_BEGIN_CONVERT_TABLE(Int32)
 	IL_EXPLICIT_OPERATOR(UInt32)
 	IL_EXPLICIT_OPERATOR(UInt64)
 	IL_EXPLICIT_OPERATOR(Char)
+	IL_UNSAFE_OPERATOR_REVERSE
 IL_END_CONVERT_TABLE
 
 /*
@@ -903,6 +933,7 @@ explicit operator short(uint x);
 explicit operator ushort(uint x);
 explicit operator int(uint x);
 explicit operator char(uint x);
+explicit operator void *(uint x);
 
 */
 
@@ -920,6 +951,7 @@ IL_BEGIN_CONVERT_TABLE(UInt32)
 	IL_EXPLICIT_OPERATOR(UInt16)
 	IL_EXPLICIT_OPERATOR(Int32)
 	IL_EXPLICIT_OPERATOR(Char)
+	IL_UNSAFE_OPERATOR_REVERSE
 IL_END_CONVERT_TABLE
 
 /*
@@ -937,6 +969,7 @@ explicit operator int(long x);
 explicit operator uint(long x);
 explicit operator ulong(long x);
 explicit operator char(long x);
+explicit operator void *(long x);
 
 */
 
@@ -954,6 +987,7 @@ IL_BEGIN_CONVERT_TABLE(Int64)
 	IL_EXPLICIT_OPERATOR(UInt32)
 	IL_EXPLICIT_OPERATOR(UInt64)
 	IL_EXPLICIT_OPERATOR(Char)
+	IL_UNSAFE_OPERATOR_REVERSE
 IL_END_CONVERT_TABLE
 
 /*
@@ -971,6 +1005,7 @@ explicit operator int(ulong x);
 explicit operator uint(ulong x);
 explicit operator long(ulong x);
 explicit operator char(ulong x);
+explicit operator void *(ulong x);
 
 */
 
@@ -988,6 +1023,7 @@ IL_BEGIN_CONVERT_TABLE(UInt64)
 	IL_EXPLICIT_OPERATOR(UInt32)
 	IL_EXPLICIT_OPERATOR(Int64)
 	IL_EXPLICIT_OPERATOR(Char)
+	IL_UNSAFE_OPERATOR_REVERSE
 IL_END_CONVERT_TABLE
 
 /*
@@ -1462,7 +1498,7 @@ const ILOperator *ILFindBinaryOperator(const ILOperator *table,
 	/* Apply numeric promotions */
 	if(!(type1->isEnum) && !(type2->isEnum))
 	{
-		if(table != ILOp_Shl && table != ILOp_Shr)
+		if(table != ILOp_Shl && table != ILOp_Shr && table != ILOp_UShr)
 		{
 			/* Arithmetic or bitwise operator */
 			if(type1 == &ILSystemDecimal)
