@@ -43,12 +43,13 @@ void CGenCloneLine(ILNode *newNode, ILNode *oldNode)
 	yysetlinenum(newNode, yygetlinenum(oldNode));
 }
 
-CSemValue CSemInlineAnalysis(ILGenInfo *info, ILNode *node, ILScope *scope)
+CSemValue CSemInlineAnalysis(ILGenInfo *info, ILNode *node,
+							 ILNode **parent, ILScope *scope)
 {
 	ILScope *currentScope = info->currentScope;
 	CSemValue result;
 	info->currentScope = scope;
-	result = ILNode_CSemAnalysis(node, info, &node, 1);
+	result = ILNode_CSemAnalysis(node, info, parent, 1);
 	info->currentScope = currentScope;
 	return result;
 }
@@ -92,30 +93,12 @@ void CGenBeginCode(ILGenInfo *info)
 	/* C doesn't have goto label scopes */
 	info->hasGotoScopes = 0;
 
-	/* Tag the module with the memory model, which tells the linker
+	/* Tag the module with "ModuleAttribute", which tells the linker
 	   that this is a C module requiring special treatment */
 	if(info->asmOutput)
 	{
-		if(CTypeAlignModifiers != 0)
-		{
-			fprintf(info->asmOutput, ".custom instance void "
-					"OpenSystem.C.MemoryModelAttribute"
-					"::.ctor(int32, int32) = "
-					"(01 00 %02X 00 00 00 %02X %02X %02X %02X 00 00)\n",
-					(int)(CTypePtrSize * 8),
-					(int)(CTypeAlignModifiers & 0xFF),
-					(int)((CTypeAlignModifiers >> 8) & 0xFF),
-					(int)((CTypeAlignModifiers >> 16) & 0xFF),
-					(int)((CTypeAlignModifiers >> 24) & 0xFF));
-		}
-		else
-		{
-			fprintf(info->asmOutput, ".custom instance void "
-					"OpenSystem.C.MemoryModelAttribute"
-					"::.ctor(int32) = "
-					"(01 00 %02X 00 00 00 00 00)\n",
-					(int)(CTypePtrSize * 8));
-		}
+		fprintf(info->asmOutput, ".custom instance void "
+				"OpenSystem.C.ModuleAttribute::.ctor() = (01 00 00 00)\n");
 	}
 
 	/* Initialize the global definition scope */
@@ -179,24 +162,21 @@ void CGenAddress(ILGenInfo *info, ILNode *node)
 
 void CGenSizeOf(ILGenInfo *info, ILType *type)
 {
-	ILUInt32 size = CTypeSizeAndAlign(type, 0);
-	if(size != CTYPE_DYNAMIC)
+	CTypeLayoutInfo layout;
+	ILClass *classInfo;
+
+	CTypeGetLayoutInfo(type, &layout);
+	if(layout.category == C_TYPECAT_FIXED)
 	{
-		ILGenUInt32(info, size);
+		/* Output the fixed size as a constant */
+		ILGenUInt32(info, layout.size);
+		ILGenAdjust(info, 1);
 	}
-	else
+	else if(layout.category == C_TYPECAT_DYNAMIC)
 	{
-		type = ILTypeStripPrefixes(type);
-		if(type == ILType_Float)
-		{
-			/* We need to use a special value type to measure
-			   native floats, because the CLS doesn't have an
-			   appropriate type that we can measure directly */
-			type = ILFindNonSystemType(info, "LongDouble", "OpenSystem.C");
-			ILGenClassToken(info, IL_OP_PREFIX + IL_PREFIX_OP_SIZEOF,
-							ILTypeToClass(info, type));
-		}
-		else if(ILType_IsPrimitive(type) || ILType_IsValueType(type))
+		/* We need to get the size dynamically from "layout.measureType" */
+		type = layout.measureType;
+		if(ILType_IsPrimitive(type) || ILType_IsValueType(type))
 		{
 			/* Calculate the size of the underlying value type */
 			ILGenClassToken(info, IL_OP_PREFIX + IL_PREFIX_OP_SIZEOF,
@@ -208,8 +188,21 @@ void CGenSizeOf(ILGenInfo *info, ILType *type)
 			ILGenClassToken(info, IL_OP_PREFIX + IL_PREFIX_OP_SIZEOF,
 						    ILTypeToClass(info, ILType_Int));
 		}
+		ILGenAdjust(info, 1);
+	}
+	else if(layout.category == C_TYPECAT_COMPLEX)
+	{
+		/* Complex types: load the read-only "size.of" field onto the stack */
+		classInfo = ILTypeToClass(info, ILTypeStripPrefixes(type));
+		if(classInfo && info->asmOutput)
+		{
+			fputs("\tldsfld\tunsigned int32 ", info->asmOutput);
+			ILDumpClassName(info->asmOutput, info->image, classInfo,
+							IL_DUMP_QUOTE_NAMES);
+			fputs("::'size.of'\n", info->asmOutput);
+		}
+		ILGenAdjust(info, 1);
 	}
-	ILGenAdjust(info, 1);
 }
 
 #ifdef	__cplusplus
