@@ -56,6 +56,8 @@ extern	"C" {
 ILGenInfo CCCodeGen;
 ILNode *CCParseTree;
 ILNode *CCParseTreeEnd;
+ILNode *CCStandaloneAttrs;
+ILNode *CCStandaloneAttrsEnd;
 ILScope *CCGlobalScope;
 
 /*
@@ -1318,6 +1320,29 @@ void CCPluginAddTopLevel(ILNode *node)
 	}
 }
 
+void CCPluginAddStandaloneAttrs(ILNode *node)
+{
+	if(!node)
+	{
+		return;
+	}
+	if(!CCStandaloneAttrs)
+	{
+		CCStandaloneAttrs = ILNode_List_create();
+		CCStandaloneAttrsEnd = 0;
+	}
+	if(!CCStandaloneAttrsEnd)
+	{
+		CCStandaloneAttrsEnd = CCStandaloneAttrs;
+	}
+	ILNode_List_Add(CCStandaloneAttrsEnd, node);
+	if(((ILNode_List *)CCStandaloneAttrsEnd)->rest)
+	{
+		CCStandaloneAttrsEnd =
+			(ILNode *)(((ILNode_List *)CCStandaloneAttrsEnd)->rest);
+	}
+}
+
 int CCLoadLibrary(const char *name)
 {
 	return LoadLibrary(name, nostdlib_flag, 1);
