@@ -94,9 +94,11 @@ extern	"C" {
 	#define	IL_MEMSET(dst,ch,len)			(ILMemSet((dst), (ch), (len)))
 	#define	IL_MEMCMP(dst,src,len)			(ILMemCmp((dst), (src), (len)))
 
-	#define X86_64_CGOTO(pc) __asm__ __volatile__ (\
+	#define X86_64_CGOTO(pc) do { __asm__ __volatile__ (\
 								"jmpq *(%0)" \
-								:: "r" (pc) )
+								:: "r" (pc) ); \
+								/* just to fool the compiler */ \
+								goto ** ((void **)(pc)); } while(0)
 	/* VM_CGOTO_SWITCH segfaults with just a jmpq */
 	#define VM_CGOTO_PREFIXSWITCH(val) X86_64_CGOTO(pc)
 	#define VM_CGOTO_BREAK(val) X86_64_CGOTO(pc)
@@ -141,6 +143,67 @@ extern	"C" {
 	#define	IL_MEMCMP(dst,src,len)			(ILMemCmp((dst), (src), (len)))
 #endif
 
+/*
+ * Defining macros to declare variables in the interpreter loop volatile.
+ * This is done to fix problems with gcc's optimized code.
+ */
+#ifdef __GNUC__
+#if (__GNUC__ == 4) && defined(CVM_X86)
+#define IL_PC_VOLATILE volatile
+#define IL_STACKTOP_VOLATILE volatile
+#define IL_FRAME_VOLATILE volatile
+#define IL_STACKMAX_VOLATILE volatile
+#define IL_METHOD_VOLATILE volatile
+#define IL_METHODTOCALL_VOLATILE volatile
+#define IL_CALLFRAME_VOLATILE volatile
+#define IL_TEMPPTR_VOLATILE volatile
+#endif
+#if (__GNUC__ == 4) && defined(CVM_X86_64)
+#define IL_PC_VOLATILE volatile
+#define IL_STACKTOP_VOLATILE volatile
+#define IL_FRAME_VOLATILE volatile
+#define IL_STACKMAX_VOLATILE volatile
+#define IL_METHOD_VOLATILE volatile
+#define IL_METHODTOCALL_VOLATILE volatile
+#define IL_CALLFRAME_VOLATILE volatile
+#define IL_TEMPPTR_VOLATILE volatile
+#endif
+#if (__GNUC__ == 4) && defined(CVM_PPC)
+#define IL_PC_VOLATILE volatile
+#define IL_STACKTOP_VOLATILE volatile
+#define IL_FRAME_VOLATILE volatile
+#define IL_STACKMAX_VOLATILE volatile
+#define IL_METHOD_VOLATILE volatile
+#define IL_METHODTOCALL_VOLATILE volatile
+#define IL_CALLFRAME_VOLATILE volatile
+#define IL_TEMPPTR_VOLATILE volatile
+#endif
+#endif
+#ifndef IL_PC_VOLATILE
+#define IL_PC_VOLATILE
+#endif
+#ifndef IL_STACKTOP_VOLATILE
+#define IL_STACKTOP_VOLATILE
+#endif
+#ifndef IL_FRAME_VOLATILE
+#define IL_FRAME_VOLATILE
+#endif
+#ifndef IL_STACKMAX_VOLATILE
+#define IL_STACKMAX_VOLATILE
+#endif
+#ifndef IL_METHOD_VOLATILE
+#define IL_METHOD_VOLATILE
+#endif
+#ifndef IL_METHODTOCALL_VOLATILE
+#define IL_METHODTOCALL_VOLATILE
+#endif
+#ifndef IL_CALLFRAME_VOLATILE
+#define IL_CALLFRAME_VOLATILE
+#endif
+#ifndef IL_TEMPPTR_VOLATILE
+#define IL_TEMPPTR_VOLATILE
+#endif
+
 #if defined(IL_USE_INTERRUPT_BASED_X)
 	#if defined(IL_INTERRUPT_HAVE_X86_CONTEXT) && defined(REGISTER_ASM_X86)
 		/* If the interrupt subsystem can provide us the x86 registers at the
@@ -368,7 +431,7 @@ static IL_INLINE ILInt64 ReadLong(CVMWord *stack)
 	return *((ILInt64 *)stack);
 #else
 	ILInt64 temp;
-	ILMemCpy(&temp, stack, sizeof(ILInt64));
+	IL_MEMCPY(&temp, stack, sizeof(ILInt64));
 	return temp;
 #endif
 }
@@ -381,7 +444,7 @@ static IL_INLINE void WriteLong(CVMWord *stack, ILInt64 value)
 #ifdef CVM_LONGS_ALIGNED_WORD
 	*((ILInt64 *)stack) = value;
 #else
-	ILMemCpy(stack, &value, sizeof(ILInt64));
+	IL_MEMCPY(stack, &value, sizeof(ILInt64));
 #endif
 }
 
@@ -394,7 +457,7 @@ static IL_INLINE ILUInt64 ReadULong(CVMWord *stack)
 	return *((ILUInt64 *)stack);
 #else
 	ILUInt64 temp;
-	ILMemCpy(&temp, stack, sizeof(ILUInt64));
+	IL_MEMCPY(&temp, stack, sizeof(ILUInt64));
 	return temp;
 #endif
 }
@@ -407,7 +470,7 @@ static IL_INLINE void WriteULong(CVMWord *stack, ILUInt64 value)
 #ifdef CVM_LONGS_ALIGNED_WORD
 	*((ILUInt64 *)stack) = value;
 #else
-	ILMemCpy(stack, &value, sizeof(ILUInt64));
+	IL_MEMCPY(stack, &value, sizeof(ILUInt64));
 #endif
 }
 
@@ -422,7 +485,7 @@ static IL_INLINE ILNativeFloat ReadFloat(CVMWord *stack)
 	return *((ILNativeFloat *)stack);
 #else
 	ILNativeFloat temp;
-	ILMemCpy(&temp, stack, sizeof(ILNativeFloat));
+	IL_MEMCPY(&temp, stack, sizeof(ILNativeFloat));
 	return temp;
 #endif
 }
@@ -435,7 +498,7 @@ static IL_INLINE void WriteFloat(CVMWord *stack, ILNativeFloat value)
 #ifdef CVM_REALS_ALIGNED_WORD
 	*((ILNativeFloat *)stack) = value;
 #else
-	ILMemCpy(stack, &value, sizeof(ILNativeFloat));
+	IL_MEMCPY(stack, &value, sizeof(ILNativeFloat));
 #endif
 }
 
@@ -448,7 +511,7 @@ static IL_INLINE ILDouble ReadDouble(CVMWord *stack)
 	return *((ILDouble *)stack);
 #else
 	ILDouble temp;
-	ILMemCpy(&temp, stack, sizeof(ILDouble));
+	IL_MEMCPY(&temp, stack, sizeof(ILDouble));
 	return temp;
 #endif
 }
@@ -461,7 +524,7 @@ static IL_INLINE void WriteDouble(CVMWord *stack, ILDouble value)
 #ifdef CVM_DOUBLES_ALIGNED_WORD
 	*((ILDouble *)stack) = value;
 #else
-	ILMemCpy(stack, &value, sizeof(ILDouble));
+	IL_MEMCPY(stack, &value, sizeof(ILDouble));
 #endif
 }
 
@@ -518,12 +581,12 @@ CVM_DEFINE_TABLES();
 
 int _ILCVMInterpreter(ILExecThread *thread)
 {
-	REGISTER_ASM_PC(unsigned char *pc);
-	REGISTER_ASM_STACK(CVMWord *stacktop);
-	REGISTER_ASM_FRAME(CVMWord *frame);
+	REGISTER_ASM_PC(unsigned char *IL_PC_VOLATILE pc);
+	REGISTER_ASM_STACK(CVMWord *IL_STACKTOP_VOLATILE stacktop);
+	REGISTER_ASM_FRAME(CVMWord *IL_FRAME_VOLATILE frame);
 	int divResult;
-	CVMWord *stackmax;
-	ILMethod *method;
+	CVMWord  *IL_STACKMAX_VOLATILE stackmax;
+	ILMethod *IL_METHOD_VOLATILE method;
 	void *nativeArgs[CVM_MAX_NATIVE_ARGS + 1];
 
 	/* Define local variables that are used by the instruction categories */
