@@ -65,7 +65,7 @@
  *
  * These macros get called whenever a function gets called and when it is left,
  * respectivly. They record the time a function needed to execute INCLUDING
- * CHILD FUNCTIONS. It's very dump and simple but better than no time profiling
+ * CHILD FUNCTIONS. It's very dumb and simple but better than no time profiling
  * at all ;-)
  */
 #define DO_PROFILE_START() \
@@ -594,8 +594,8 @@ ILUInt32 _ILGetMethodParamCount(ILExecThread *thread, ILMethod *method,
 
 #elif defined(IL_CVM_LOCALS)
 
-ILMethod *methodToCall;
-ILCallFrame *callFrame = 0;
+ILMethod *IL_METHODTOCALL_VOLATILE methodToCall;
+ILCallFrame *IL_CALLFRAME_VOLATILE callFrame = 0;
 
 #elif defined(IL_CVM_MAIN)
 
@@ -1093,6 +1093,18 @@ VMCASE(COP_CALL_INTERFACE):
 				MISSING_METHOD_EXCEPTION();
 			}
 		}
+		#if IL_DEBUG_IMTS
+		if(methodToCall)
+		{
+			fprintf(stderr, "%s:%d found <%s:%s> for <%s:%s> \n", 
+				__FILE__, __LINE__,
+				ILClass_Name(methodToCall->member.owner),
+				ILMethod_Name(methodToCall),
+				ILClass_Name(
+					ILMethod_Owner(CVM_ARG_DWIDE_PTR_SMALL(ILMethod *))),
+				ILMethod_Name(CVM_ARG_DWIDE_PTR_SMALL(ILMethod*)));
+		}
+		#endif
 	#else
 		methodToCall = _ILLookupInterfaceMethod
 			(GetObjectClassPrivate(tempptr), CVM_ARG_DWIDE_PTR_SMALL(ILClass *),
@@ -1231,9 +1243,10 @@ popFrame:
 
 	CHECK_MANAGED_BARRIER();
 	
-	DO_PROFILE_STOP();
-
 	callFrame = &(thread->frameStack[--(thread->numFrames)]);
+
+	DO_PROFILE_STOP();
+	
 	methodToCall = callFrame->method;
 	pc = callFrame->pc;
 	thread->exceptHeight = callFrame->exceptHeight;
@@ -2029,7 +2042,7 @@ VMCASE(COP_PREFIX_PACK_VARARGS):
 	COPY_STATE_TO_THREAD();
 	tempNum = _ILPackCVMStackArgs
 		(thread, stacktop, CVMP_ARG_WORD, CVMP_ARG_WORD2,
-		 CVMP_ARG_WORD2_PTR(ILType *), &tempptr);
+		 CVMP_ARG_WORD2_PTR(ILType *), (void **)&tempptr);
 	RESTORE_STATE_FROM_THREAD();
 
 	stacktop -= tempNum;
@@ -2059,7 +2072,18 @@ VMBREAK(COP_PREFIX_PACK_VARARGS);
  */
 VMCASE(COP_PREFIX_PROFILE_COUNT):
 {
-	ILInterlockedIncrement(&method->count);
+#ifdef ENHANCED_PROFILER
+	/* If the enhanced profiler is selected then don't count when
+	 * profiling is disabled
+	 * (e.g. via DotGNU.Misc.Profiling.StopProfiling())
+	 */
+	if ((thread->profilingEnabled) && ((ILCoderGetFlags (thread->process->coder) & IL_CODER_FLAG_METHOD_PROFILE) != 0))
+	{
+#endif
+		ILInterlockedIncrement(&method->count);
+#ifdef ENHANCED_PROFILER
+	}
+#endif
 	MODIFY_PC_AND_STACK(CVMP_LEN_NONE,0);
 }
 VMBREAK(COP_PREFIX_PROFILE_COUNT);
