@@ -190,11 +190,14 @@ static IL_INLINE int F2LUOvf(CVMWord *posn)
 
 #endif /* IL_CONFIG_FP_SUPPORTED */
 
+#ifdef IL_CONFIG_PINVOKE
+
 /*
  * Convert a reference to a string array into a pointer to a C array.
  */
 static void *RefArrayToC(ILExecThread *thread, void *ref,
-						 char *(*conv)(ILExecThread *thread, ILString *str))
+						 char *(*conv)(ILExecThread *thread, ILString *str),
+						 int needRefPtr)
 {
 	void *result;
 	System_Array *array;
@@ -214,7 +217,14 @@ static void *RefArrayToC(ILExecThread *thread, void *ref,
 	}
 
 	/* Extract the string array and then create a new C array */
-	array = *((System_Array **)ref);
+	if(needRefPtr)
+	{
+		array = *((System_Array **)ref);
+	}
+	else
+	{
+		array = (System_Array *)ref;
+	}
 	result = ILGCAlloc(sizeof(void *) * (array->length + 2));
 	if(!result)
 	{
@@ -240,9 +250,18 @@ static void *RefArrayToC(ILExecThread *thread, void *ref,
 	*newArray = 0;
 
 	/* Return the new reference to the caller */
-	return result;
+	if(needRefPtr)
+	{
+		return result;
+	}
+	else
+	{
+		return (void *)(((void **)result) + 1);
+	}
 }
 
+#endif /* IL_CONFIG_PINVOKE */
+
 #elif defined(IL_CVM_LOCALS)
 
 ILInt32 position;
@@ -1910,6 +1929,80 @@ VMCASE(COP_PREFIX_UTF82STR):
 }
 VMBREAK(COP_PREFIX_UTF82STR);
 
+/**
+ * <opcode name="str2utf16" group="Conversion operators">
+ *   <operation>Convert <code>string</code> to <code>utf16 char *</code>
+ *              </operation>
+ *
+ *   <format>prefix<fsep/>str2utf16</format>
+ *   <dformat>{str2utf16}</dformat>
+ *
+ *   <form name="str2utf16" code="COP_PREFIX_STR2UTF16"/>
+ *
+ *   <before>..., value</before>
+ *   <after>..., result</after>
+ *
+ *   <description>The <i>value</i> is popped from the stack as
+ *   type <code>string</code>.  The string is converted into a
+ *   <i>result</i> wide character buffer using the UTF-16 encoding.
+ *   A pointer to the buffer is pushed onto the stack as type
+ *   <code>ptr</code>.</description>
+ *
+ *   <notes>This instruction is used to convert C# strings into wide
+ *   character buffers during "PInvoke" marshalling operations.</notes>
+ * </opcode>
+ */
+VMCASE(COP_PREFIX_STR2UTF16):
+{
+	/* Convert a string object into a UTF-16 character buffer */
+	if(stacktop[-1].ptrValue)
+	{
+		COPY_STATE_TO_THREAD();
+		stacktop[-1].ptrValue =
+			(void *)ILStringToUTF16
+				(thread, (ILString *)(stacktop[-1].ptrValue));
+		RESTORE_STATE_FROM_THREAD();
+	}
+	MODIFY_PC_AND_STACK(CVMP_LEN_NONE, 0);
+}
+VMBREAK(COP_PREFIX_STR2UTF16);
+
+/**
+ * <opcode name="utf162str" group="Conversion operators">
+ *   <operation>Convert <code>utf16 char *</code> to <code>string</code>
+ *              </operation>
+ *
+ *   <format>prefix<fsep/>utf162str</format>
+ *   <dformat>{utf162str}</dformat>
+ *
+ *   <form name="utf162str" code="COP_PREFIX_UTF162STR"/>
+ *
+ *   <before>..., value</before>
+ *   <after>..., result</after>
+ *
+ *   <description>The <i>value</i> is popped from the stack as
+ *   type <code>utf16 char *</code>.  The wide character buffer
+ *   is converted into a <i>result</i> of type <code>string</code>,
+ *   which is pushed onto the stack.</description>
+ *
+ *   <notes>This instruction is used to convert wide character buffers
+ *   into C# strings during "PInvoke" marshalling operations.</notes>
+ * </opcode>
+ */
+VMCASE(COP_PREFIX_UTF162STR):
+{
+	/* Convert a UTF-16 character buffer into a string */
+	if(stacktop[-1].ptrValue)
+	{
+		COPY_STATE_TO_THREAD();
+		stacktop[-1].ptrValue = (void *)ILStringWCreate
+			(thread, (const ILUInt16 *)(stacktop[-1].ptrValue));
+		RESTORE_STATE_FROM_THREAD();
+	}
+	MODIFY_PC_AND_STACK(CVMP_LEN_NONE, 0);
+}
+VMBREAK(COP_PREFIX_UTF162STR);
+
 /**
  * <opcode name="delegate2fnptr" group="Conversion operators">
  *   <operation>Convert a delegate into a function pointer</operation>
@@ -1936,7 +2029,7 @@ VMCASE(COP_PREFIX_DELEGATE2FNPTR):
 {
 	/* Convert a delegate into a function pointer */
 	stacktop[-1].ptrValue = _ILDelegateGetClosure
-		((ILObject *)(stacktop[-1].ptrValue));
+		(thread, (ILObject *)(stacktop[-1].ptrValue));
 	MODIFY_PC_AND_STACK(CVMP_LEN_NONE, 0);
 }
 VMBREAK(COP_PREFIX_DELEGATE2FNPTR);
@@ -2007,7 +2100,7 @@ VMCASE(COP_PREFIX_REFARRAY2ANSI):
 	/* Convert a reference to a string array into an ANSI array */
 	COPY_STATE_TO_THREAD();
 	stacktop[-1].ptrValue = RefArrayToC(thread, stacktop[-1].ptrValue,
-										ILStringToAnsi);
+										ILStringToAnsi, 1);
 	RESTORE_STATE_FROM_THREAD();
 	MODIFY_PC_AND_STACK(CVMP_LEN_NONE, 0);
 }
@@ -2045,28 +2138,222 @@ VMCASE(COP_PREFIX_REFARRAY2UTF8):
 	/* Convert a reference to a string array into a UTF-8 array */
 	COPY_STATE_TO_THREAD();
 	stacktop[-1].ptrValue = RefArrayToC(thread, stacktop[-1].ptrValue,
-										ILStringToUTF8);
+										ILStringToUTF8, 1);
 	RESTORE_STATE_FROM_THREAD();
 	MODIFY_PC_AND_STACK(CVMP_LEN_NONE, 0);
 }
 VMBREAK(COP_PREFIX_REFARRAY2UTF8);
 
+/**
+ * <opcode name="array2ansi" group="Conversion operators">
+ *   <operation>Convert an array of strings into an array of
+ *              <code>ansi char *</code> values</operation>
+ *
+ *   <format>prefix<fsep/>array2ansi</format>
+ *   <dformat>{array2ansi}</dformat>
+ *
+ *   <form name="array2ansi" code="COP_PREFIX_ARRAY2ANSI"/>
+ *
+ *   <before>..., value</before>
+ *   <after>..., result</after>
+ *
+ *   <description>The <i>value</i> is popped from the stack as
+ *   type <code>ptr</code>.  The <code>String[]</code> object at
+ *   the address <i>value</i> is retrieved.  It is converted into
+ *   a NULL-terminated C array of the same size, with all of the
+ *   strings converted into the ANSI character encoding.  Then
+ *   a pointer to this array is pushed as <i>result</i>.
+ *   </description>
+ *
+ *   <notes>This instruction is used to marshal parameters of type
+ *   <code>String[]</code> "PInvoke" marshalling operations.</notes>
+ * </opcode>
+ */
+VMCASE(COP_PREFIX_ARRAY2ANSI):
+{
+	/* Convert a reference to a string array into an ANSI array */
+	COPY_STATE_TO_THREAD();
+	stacktop[-1].ptrValue = RefArrayToC(thread, stacktop[-1].ptrValue,
+									    ILStringToAnsi, 0);
+	RESTORE_STATE_FROM_THREAD();
+	MODIFY_PC_AND_STACK(CVMP_LEN_NONE, 0);
+}
+VMBREAK(COP_PREFIX_ARRAY2ANSI);
+
+/**
+ * <opcode name="array2utf8" group="Conversion operators">
+ *   <operation>Convert an array of strings into an array of
+ *              <code>utf8 char *</code> values</operation>
+ *
+ *   <format>prefix<fsep/>array2utf8</format>
+ *   <dformat>{array2utf8}</dformat>
+ *
+ *   <form name="array2utf8" code="COP_PREFIX_ARRAY2UTF8"/>
+ *
+ *   <before>..., value</before>
+ *   <after>..., result</after>
+ *
+ *   <description>The <i>value</i> is popped from the stack as
+ *   type <code>ptr</code>.  The <code>String[]</code> object at
+ *   the address <i>value</i> is retrieved.  It is converted into
+ *   a NULL-terminated C array of the same size, with all of the
+ *   strings converted into the UTF8 character encoding.  Then
+ *   a pointer to this array is pushed as <i>result</i>.
+ *   </description>
+ *
+ *   <notes>This instruction is used to marshal parameters of type
+ *   <code>String[]</code> "PInvoke" marshalling operations.</notes>
+ * </opcode>
+ */
+VMCASE(COP_PREFIX_ARRAY2UTF8):
+{
+	/* Convert a reference to a string array into an ANSI array */
+	COPY_STATE_TO_THREAD();
+	stacktop[-1].ptrValue = RefArrayToC(thread, stacktop[-1].ptrValue,
+									    ILStringToUTF8, 0);
+	RESTORE_STATE_FROM_THREAD();
+	MODIFY_PC_AND_STACK(CVMP_LEN_NONE, 0);
+}
+VMBREAK(COP_PREFIX_ARRAY2UTF8);
+
+/**
+ * <opcode name="tocustom" group="Conversion operators">
+ *   <operation>Convert an object reference into a custom native
+ *				pointer</operation>
+ *
+ *   <format>prefix<fsep/>tocustom<fsep/>len1[4]<fsep/>len2[4]<fsep/>name1<fsep/>name2</format>
+ *   <dformat>{tocustom}<fsep/>len1<fsep/>len2<fsep/>name1<fsep/>name2</dformat>
+ *
+ *   <form name="tocustom" code="COP_PREFIX_TOCUSTOM"/>
+ *
+ *   <before>..., value</before>
+ *   <after>..., result</after>
+ *
+ *   <description>The <i>value</i> is popped from the stack as
+ *   type <code>ptr</code>.  The custom marshaler called <i>name1</i>,
+ *   with cookie <i>name2</i> is used to convert <i>value</i> into a
+ *   native pointer <i>result</i>.  The values <i>len1</i> and <i>len2</i>
+ *   are the lengths of <i>name1</i> and <i>name2</i>.
+ *   </description>
+ *
+ *   <notes>This instruction is used to perform custom marshaling
+ *   during "PInvoke" operations.</notes>
+ * </opcode>
+ */
+VMCASE(COP_PREFIX_TOCUSTOM):
+{
+	/* Convert an object reference into a native pointer */
+	COPY_STATE_TO_THREAD();
+	stacktop[-1].ptrValue = _ILObjectToCustom
+		(thread, (ILObject *)(stacktop[-1].ptrValue),
+		 CVMP_ARG_WORD2_PTR(const char *), CVMP_ARG_WORD,
+		 CVMP_ARG_WORD2_PTR2(const char *), CVMP_ARG_WORD2);
+	RESTORE_STATE_FROM_THREAD();
+	MODIFY_PC_AND_STACK(CVMP_LEN_WORD2_PTR2, 0);
+}
+VMBREAK(COP_PREFIX_TOCUSTOM);
+
+/**
+ * <opcode name="fromcustom" group="Conversion operators">
+ *   <operation>Convert a custom native pointer into an object
+ *				reference</operation>
+ *
+ *   <format>prefix<fsep/>fromcustom<fsep/>len1[4]<fsep/>len2[4]<fsep/>name1<fsep/>name2</format>
+ *   <dformat>{fromcustom}<fsep/>len1<fsep/>len2<fsep/>name1<fsep/>name2</dformat>
+ *
+ *   <form name="fromcustom" code="COP_PREFIX_FROMCUSTOM"/>
+ *
+ *   <before>..., value</before>
+ *   <after>..., result</after>
+ *
+ *   <description>The <i>value</i> is popped from the stack as
+ *   type <code>ptr</code>.  The custom marshaler called <i>name1</i>,
+ *   with cookie <i>name2</i>, is used to convert <i>value</i> into an
+ *   object reference <i>result</i>.  The values <i>len1</i> and <i>len2</i>
+ *   are the lengths of <i>name1</i> and <i>name2</i>.
+ *   </description>
+ *
+ *   <notes>This instruction is used to perform custom marshaling
+ *   during "PInvoke" operations.</notes>
+ * </opcode>
+ */
+VMCASE(COP_PREFIX_FROMCUSTOM):
+{
+	/* Convert a native pointer into an object reference */
+	COPY_STATE_TO_THREAD();
+	stacktop[-1].ptrValue = _ILCustomToObject
+		(thread, stacktop[-1].ptrValue,
+		 CVMP_ARG_WORD2_PTR(const char *), CVMP_ARG_WORD,
+		 CVMP_ARG_WORD2_PTR2(const char *), CVMP_ARG_WORD2);
+	RESTORE_STATE_FROM_THREAD();
+	MODIFY_PC_AND_STACK(CVMP_LEN_WORD2_PTR2, 0);
+}
+VMBREAK(COP_PREFIX_FROMCUSTOM);
+
+/**
+ * <opcode name="struct2native" group="Conversion operators">
+ *   <operation>Convert a struct into its native form</operation>
+ *
+ *   <format>prefix<fsep/>struct2native<fsep/>type</format>
+ *   <dformat>{struct2native}<fsep/>type</dformat>
+ *
+ *   <form name="struct2native" code="COP_PREFIX_STRUCT2NATIVE"/>
+ *
+ *   <before>..., ptr</before>
+ *   <after>...</after>
+ *
+ *   <description>The structured value at <i>ptr</i> is converted from
+ *   its managed form into its native form, converting field values
+ *   as appropriate.</description>
+ * </opcode>
+ */
+VMCASE(COP_PREFIX_STRUCT2NATIVE):
+{
+	/* Convert a struct into its native form */
+	extern void _ILStructToNative
+		(ILExecThread *thread, void *value, ILType *type);
+	COPY_STATE_TO_THREAD();
+	_ILStructToNative(thread, stacktop[-1].ptrValue, CVMP_ARG_PTR(ILType *));
+	RESTORE_STATE_FROM_THREAD();
+	MODIFY_PC_AND_STACK(CVMP_LEN_PTR, -1);
+}
+VMBREAK(COP_PREFIX_STRUCT2NATIVE);
+
 #else /* !IL_CONFIG_PINVOKE */
 
 VMCASE(COP_PREFIX_STR2ANSI):
 VMCASE(COP_PREFIX_STR2UTF8):
 VMCASE(COP_PREFIX_ANSI2STR):
 VMCASE(COP_PREFIX_UTF82STR):
+VMCASE(COP_PREFIX_STR2UTF16):
+VMCASE(COP_PREFIX_UTF162STR):
 VMCASE(COP_PREFIX_DELEGATE2FNPTR):
 VMCASE(COP_PREFIX_ARRAY2PTR):
 VMCASE(COP_PREFIX_REFARRAY2ANSI):
 VMCASE(COP_PREFIX_REFARRAY2UTF8):
+VMCASE(COP_PREFIX_ARRAY2ANSI):
+VMCASE(COP_PREFIX_ARRAY2UTF8):
 {
 	/* Stub out PInvoke-related CVM opcodes */
 	MODIFY_PC_AND_STACK(CVMP_LEN_NONE, 0);
 }
 VMBREAK(COP_PREFIX_STR2ANSI);
 
+VMCASE(COP_PREFIX_TOCUSTOM):
+VMCASE(COP_PREFIX_FROMCUSTOM):
+{
+	/* Stub out PInvoke-related CVM opcodes */
+	MODIFY_PC_AND_STACK(CVMP_LEN_WORD_PTR, 0);
+}
+VMBREAK(COP_PREFIX_TOCUSTOM);
+
+VMCASE(COP_PREFIX_STRUCT2NATIVE):
+{
+	/* Stub out PInvoke-related CVM opcodes */
+	MODIFY_PC_AND_STACK(CVMP_LEN_PTR, -1);
+}
+VMBREAK(COP_PREFIX_STRUCT2NATIVE);
+
 #endif /* !IL_CONFIG_PINVOKE */
 
 /**
