@@ -103,6 +103,10 @@ extern	"C" {
 	/*#define	IL_USE_IMTS	1*/
 #endif
 
+/*#define	IL_USE_IMTS	1 */
+
+/*#define IL_DEBUG_IMTS 1 */
+
 #if !defined(IL_CONFIG_REDUCE_CODE) && !defined(IL_WITHOUT_TOOLS)
 /* Global lock for trace outputs */
 extern ILMutex *globalTraceMutex;
@@ -166,6 +170,36 @@ struct __tagILFinalizationContext
 	ILExecProcess *volatile process;
 };
 
+#ifdef IL_CONFIG_APPDOMAINS
+/*
+ * Structure to save the old execution context of an ILExecThread
+ * when a process switch occures.
+ */
+typedef struct _tagILExecContext ILExecContext;
+struct _tagILExecContext
+{
+	/* Pointer to the previous exec context */
+	ILExecContext *prevContext;
+
+	/* Back-pointer to the process this thread belongs to */
+	ILExecProcess  *process;
+
+	/* System.Threading.Thread object */
+	ILObject *clrThread;
+
+	/* Thread-static values for this thread */
+	void		  **threadStaticSlots;
+	ILUInt32		threadStaticSlotsUsed;
+};
+#endif
+
+/* class private data */
+typedef struct _tagILClassPrivate ILClassPrivate;
+ 
+#ifdef IL_USE_JIT
+#include "jitc.h"
+#endif
+
 /*
  * Execution control context for a process.
  */
@@ -223,6 +257,11 @@ struct _tagILExecProcess
 	/* Hash table that contains all intern'ed strings within the system */
 	void		   *internHash;
 
+	/* linked list of class private data */
+	/* This is supposed to prevent them to be collected */
+	/* when they are alloceted using GC_MALLOC. */
+	ILClassPrivate *firstClassPrivate;
+
 #ifdef IL_CONFIG_USE_THIN_LOCKS
 	/* Hash table that contains all monitors */
 	void			*monitorTable;
@@ -385,6 +424,12 @@ struct _tagILExecThread
 	int		profilingEnabled;
 #endif
 
+#ifdef IL_CONFIG_APPDOMAINS
+	/* Keep track of the process switches for this thread */
+	/* Needed to clean them up when the thread is destroyed. */
+	ILExecContext *prevContext;
+#endif
+
 #if defined(IL_INTERRUPT_SUPPORTS_ILLEGAL_MEMORY_ACCESS)
 	/* Context for the current interrupt */
 	ILInterruptContext	interruptContext;
@@ -416,7 +461,6 @@ struct _tagILImplPrivate
 /*
  * Private information that is associated with a class.
  */
-typedef struct _tagILClassPrivate ILClassPrivate;
 struct _tagILClassPrivate
 {
 	ILClass		   *classInfo;			/* Back-pointer to the class */
@@ -435,10 +479,20 @@ struct _tagILClassPrivate
 	ILObject       *staticData;			/* Static data area object */
 	ILImplPrivate  *implements;			/* Interface implementation records */
 	ILNativeInt		gcTypeDescriptor;	/* Describes the layout of the type for the GC */
+	ILClassPrivate *nextClassPrivate;	/* linked list of ILClassPrivate objects */
+	ILExecProcess  *process;			/* Back-pointer to the process this class belongs to */
+#ifdef IL_USE_JIT
+	void		  **jitVtable;			/* table with vtable pointers to the vtable methods. */
+	ILJitTypes		jitTypes;			/* jit types for this CLR type */
+#endif
 #ifdef IL_USE_IMTS
 	ILUInt32		imtBase;			/* Base for IMT identifiers */
+#ifdef IL_USE_JIT
+	void		   *imt[IL_IMT_SIZE];	/* Interface method table with vtable pointers. */
+#else
 	ILMethod	   *imt[IL_IMT_SIZE];	/* Interface method table */
 #endif
+#endif
 
 };
 
@@ -480,6 +534,61 @@ void ILExecEngineDestroy(ILExecEngine *engine);
  * Returns the ILExecEngine or 0 if the function fails.
  */
 ILExecEngine *ILExecEngineCreate();
+
+/*
+ * Let the thread return from an other ILExecProcess and restore the saved
+ * state.
+ * Returns 0 on failure.
+ */
+int ILExecThreadReturnToProcess(ILExecThread *thread, ILExecContext *context);
+
+/*
+ * Let the thread join an other ILExecProcess and save the current state
+ * in context.
+ * Returns 0 on failure.
+ */
+int ILExecThreadSwitchToProcess(ILExecThread *thread,
+								ILExecProcess *process,
+								ILExecContext *context);
+
+#define IL_BEGIN_EXECPROCESS_SWITCH(thread, process) \
+{ \
+	int __processSwitched = 0; \
+	int __error = 0; \
+	ILExecContext __context; \
+	if((thread)->process != (process)) \
+	{ \
+		__processSwitched = ILExecThreadSwitchToProcess((thread), \
+														(process), \
+														 &__context); \
+	} \
+	if(!__error) \
+	{
+#define IL_END_EXECPROCESS_SWITCH(thread) \
+		if(__processSwitched) \
+		{ \
+			if(!ILExecThreadReturnToProcess((thread), &__context)) \
+			{ \
+			  \
+			} \
+		} \
+	} \
+	else \
+	{ \
+	} \
+}
+#else
+#define IL_BEGIN_EXECPROCESS_SWITCH(thread, process) \
+{
+#define IL_END_EXECPROCESS_SWITCH(thread) \
+}
+#endif
+
+#ifdef IL_USE_JIT
+/*
+ * Class information for the JIT coder.
+ */
+extern ILCoderClass const _ILJITCoderClass;
 #endif
 
 /*
@@ -491,15 +600,25 @@ extern ILCoderClass const _ILCVMCoderClass;
  * Pack parameters onto the CVM stack for a call, using a "va_list"
  * as the source of the values.
  */
+#ifdef IL_USE_JIT
+int _ILCallPackVaParams(ILExecThread *thread, ILType *signature,
+					    void *argBuffer, void **jitArgs, void *userData);
+#else
 int _ILCallPackVaParams(ILExecThread *thread, ILMethod *method,
 					    int isCtor, void *_this, void *userData);
+#endif
 
 /*
  * Pack parameters onto the CVM stack for a call, using an array
  * of ILExecValue values to supply the parameters.
  */
+#ifdef IL_USE_JIT
+int _ILCallPackVParams(ILExecThread *thread, ILType *signature,
+					   void *argBuffer, void **jitArgs, void *userData);
+#else
 int _ILCallPackVParams(ILExecThread *thread, ILMethod *method,
 					   int isCtor, void *_this, void *userData);
+#endif
 
 /*
  * Unpack a method result from the CVM stack and store it into
@@ -518,8 +637,13 @@ void _ILCallUnpackVResult(ILExecThread *thread, ILMethod *method,
 /*
  * Prototype for a parameter packing function.
  */
+#ifdef IL_USE_JIT
+typedef int (*ILCallPackFunc)(ILExecThread *thread, ILType *signature,
+					          void *argBuffer, void **jitArgs, void *userData);
+#else
 typedef int (*ILCallPackFunc)(ILExecThread *thread, ILMethod *method,
 					          int isCtor, void *_this, void *userData);
+#endif
 
 /*
  * Prototype for a return value unpacking function.
@@ -545,12 +669,13 @@ int _ILCVMInterpreter(ILExecThread *thread);
  * Lay out a class's fields, virtual methods, and interfaces.
  * Returns zero if there is something wrong with the definition.
  */
-int _ILLayoutClass(ILClass *info);
+int _ILLayoutClass(ILExecProcess *process, ILClass *info);
 
 /*
  * Lay out a class and return its size and alignment.
  */
-ILUInt32 _ILLayoutClassReturn(ILClass *info, ILUInt32 *alignment);
+ILUInt32 _ILLayoutClassReturn(ILExecProcess *process, ILClass *info,
+								ILUInt32 *alignment);
 
 /*
  * Determine if layout of a class has already been done.
@@ -568,7 +693,8 @@ int _ILVerify(ILCoder *coder, unsigned char **start, ILMethod *method,
  * call a PInvoke or "internalcall" method.  Returns NULL
  * if insufficient memory for the structure.
  */
-void *_ILMakeCifForMethod(ILMethod *method, int isInternal);
+void *_ILMakeCifForMethod(ILExecProcess *process, ILMethod *method,
+							int isInternal);
 
 /*
  * Construct the "ffi_cif" structure that is needed to
@@ -576,13 +702,15 @@ void *_ILMakeCifForMethod(ILMethod *method, int isInternal);
  * allocation mode.  Returns NULL if insufficient memory
  * for the structure.
  */
-void *_ILMakeCifForConstructor(ILMethod *method, int isInternal);
+void *_ILMakeCifForConstructor(ILExecProcess *process, ILMethod *method,
+								int isInternal);
 
 /*
  * Make a native closure for a particular delegate.  "method"
  * is the method within the delegate object.
  */
-void *_ILMakeClosureForDelegate(ILObject *delegate, ILMethod *method);
+void *_ILMakeClosureForDelegate(ILExecProcess *process, ILObject *delegate,
+								ILMethod *method);
 
 /*
  * Convert a method into executable code.  Returns a pointer
@@ -754,7 +882,7 @@ int _ILUnrollMethod(ILExecThread *thread, ILCoder *coder,
  * Determine the size of a type's values in bytes.  This assumes
  * that the caller has obtained the metadata write lock.
  */
-ILUInt32 _ILSizeOfTypeLocked(ILType *type);
+ILUInt32 _ILSizeOfTypeLocked(ILExecProcess *process, ILType *type);
 
 /*
  * Get the native closure associated with a delegate.  Returns NULL
@@ -769,18 +897,18 @@ ILUInt32 _ILGetMethodParamCount(ILExecThread *thread, ILMethod *method,
 								int suppressThis);
 
 /*
- * Lock metadata for reading or writing from the current thread.
+ * Lock metadata for reading or writing from the given process.
  */
-#define	IL_METADATA_WRLOCK(thread)	\
-			ILRWLockWriteLock((thread)->process->metadataLock)
-#define	IL_METADATA_RDLOCK(thread)	\
-			ILRWLockReadLock((thread)->process->metadataLock)
+#define	IL_METADATA_WRLOCK(process)	\
+			ILRWLockWriteLock((process)->metadataLock)
+#define	IL_METADATA_RDLOCK(process)	\
+			ILRWLockReadLock((process)->metadataLock)
 
 /*
- * Unlock metadata from the current thread.
+ * Unlock metadata from the given process.
  */
-#define	IL_METADATA_UNLOCK(thread)	\
-			ILRWLockUnlock((thread)->process->metadataLock)
+#define	IL_METADATA_UNLOCK(process)	\
+			ILRWLockUnlock((process)->metadataLock)
 
 #ifdef IL_CONFIG_DEBUG_LINES
 
@@ -812,6 +940,12 @@ ILObject *_ILCustomToObject(ILExecThread *thread, void *ptr,
 							const char *customName, int customNameLen,
 							const char *customCookie, int customCookieLen);
 
+/*
+ * Get an ILExecThread from the given support thread.
+ */
+#define _ILExecThreadFromThread(thread) \
+	((ILExecThread *)(ILThreadGetObject(thread)))
+
 /*
  *	Gets the managed thread object from an engine thread.
  */
@@ -892,6 +1026,11 @@ ILInt32 _ILExecThreadGetState(ILExecThread *thread, ILThread* supportThread);
  */
 void _ILExecThreadResumeThread(ILExecThread *thread, ILThread *supportThread);
 
+/*
+ * Get the ILExecProcess in which a thread is running.
+ */
+#define _ILExecThreadProcess(thread) ((thread)->process)
+
 /*
  * Creates a monitor used by the execution engine.
  * The monitor is created on the GC heap and will be automatically
