@@ -45,25 +45,41 @@ typedef struct
 	int			hasFinalizer;
 	int			managedInstance;
 	int			managedStatic;
-
+#ifdef IL_USE_JIT
+	void	  **jitVtable;
+	ILJitTypes *jitTypes;
+#endif	
 } LayoutInfo;
 
 /*
  * Forward declaration.
  */
-static int LayoutClass(ILClass *info, LayoutInfo *layout);
+static int LayoutClass(ILExecProcess *process, ILClass *info, LayoutInfo *layout);
 
 /*
  * Get the layout information for a type.  Returns zero
  * if there is something wrong with the type.
  */
-static int LayoutType(ILType *type, LayoutInfo *layout)
+static int LayoutType(ILExecProcess *process, ILType *type, LayoutInfo *layout)
 {
 	type = ILTypeStripPrefixes(type);
 	if(ILType_IsPrimitive(type))
 	{
 		/* Lay out a primitive type */
 		layout->managedInstance = 0;
+	#ifdef IL_USE_JIT
+		if(!(layout->jitTypes = ILJitPrimitiveClrTypeToJitTypes(ILType_ToElement(type))))
+		{
+			return 0;
+		}
+		layout->size = ILJitTypeGetSize(layout->jitTypes->jitTypeBase);
+		layout->alignment = ILJitTypeGetAlignment(layout->jitTypes->jitTypeBase);
+		if(ILType_ToElement(type) == IL_META_ELEMTYPE_TYPEDBYREF)
+		{
+			layout->managedInstance = 1;
+		}
+		layout->jitVtable = 0;
+	#else
 		switch(ILType_ToElement(type))
 		{
 			case IL_META_ELEMTYPE_BOOLEAN:
@@ -169,6 +185,7 @@ static int LayoutType(ILType *type, LayoutInfo *layout)
 
 			default: return 0;
 		}
+	#endif /* IL_USE_JIT */
 		layout->nativeSize = layout->size;
 		layout->nativeAlignment = layout->alignment;
 		layout->vtableSize = 0;
@@ -186,16 +203,25 @@ static int LayoutType(ILType *type, LayoutInfo *layout)
 		ILType *synType = ILClassGetSynType(classInfo);
 		if(synType == 0)
 		{
-			return LayoutClass(classInfo, layout);
+			return LayoutClass(process, classInfo, layout);
 		}
 		else
 		{
-			return LayoutType(ILTypeStripPrefixes(synType), layout);
+			return LayoutType(process, ILTypeStripPrefixes(synType), layout);
 		}
 	}
 	else
 	{
 		/* Everything else is laid out as a pointer */
+	#ifdef IL_USE_JIT
+		if(!(layout->jitTypes = ILJitPrimitiveClrTypeToJitTypes(IL_META_ELEMTYPE_TYPEDBYREF)))
+		{
+			return 0;
+		}
+		layout->size = ILJitTypeGetSize(layout->jitTypes->jitTypeBase);
+		layout->alignment = ILJitTypeGetAlignment(layout->jitTypes->jitTypeBase);
+		layout->jitVtable = 0;
+	#else
 	#if defined(HAVE_LIBFFI)
 		layout->size = ffi_type_pointer.size;
 		layout->alignment = ffi_type_pointer.alignment;
@@ -203,6 +229,7 @@ static int LayoutType(ILType *type, LayoutInfo *layout)
 		layout->size = sizeof(void *);
 		layout->alignment = _IL_ALIGN_FOR_TYPE(void_p);
 	#endif
+	#endif /* IL_USE_JIT */
 		layout->nativeSize = layout->size;
 		layout->nativeAlignment = layout->alignment;
 		layout->vtableSize = 0;
@@ -452,24 +479,16 @@ static int ComputeInterfaceTable(ILClass *info, ILClass *interface)
  * This is necessary because we convert methods one at a time, instead of
  * class at a time like RVM does.
  */
-static void BuildIMT(ILClass *info, ILClassPrivate *classPrivate)
+static void BuildIMT(ILExecProcess *process, ILClass *info,
+					 ILClassPrivate *classPrivate)
 {
-	ILExecThread *thread;
-	ILExecProcess *process;
 	ILImplPrivate *impl;
 	ILClassPrivate *implPrivate;
+	ILClassPrivate *parentPrivate;
 	ILUInt32 posn, size;
 	ILUInt32 vtableIndex;
 	ILUInt32 imtIndex;
 
-	/* Find the process that owns the class */
-	thread = ILExecThreadCurrent();
-	if(!thread)
-	{
-		return;
-	}
-	process = thread->process;
-
 	/* Is this class itself an interface? */
 	if(ILClass_IsInterface(info))
 	{
@@ -486,58 +505,104 @@ static void BuildIMT(ILClass *info, ILClassPrivate *classPrivate)
 		return;
 	}
 
-	/* Process the implementation records that are attached to this class */
-	impl = classPrivate->implements;
-	while(impl != 0)
+	if(ILClass_IsAbstract(info))
 	{
-		implPrivate = (ILClassPrivate *)(impl->interface->userData);
-		if(implPrivate)
-		{
-			/* Allocate a base identifer for the interface if necessary.
-			   This will probably never be used because interfaces are
-			   typically laid out before their implementing classes, and
-			   the allocation above will catch us.  But let's be paranoid
-			   and allocate the base identifier here anyway, just in case */
-			size = (ILUInt32)(implPrivate->vtableSize);
-			if(!(implPrivate->imtBase))
-			{
-				implPrivate->imtBase = process->imtBase;
-				process->imtBase += size;
-			}
+		/* will never have an object */
+		return;
+	}
 
-			/* Process the members of this interface */
-			for(posn = 0; posn < size; ++posn)
+	parentPrivate = classPrivate;
+	#if IL_DEBUG_IMTS
+	fprintf(stderr, "%s->imt = {\n", ILClass_Name(info));
+	#endif
+	while(parentPrivate != NULL)
+	{
+		/* Process the implementation records that are attached to this class */
+		impl = parentPrivate->implements;
+		while(impl != 0)
+		{
+			implPrivate = (ILClassPrivate *)(impl->interface->userData);
+			#if IL_DEBUG_IMTS
+			fprintf(stderr, "\t /* %s */\n", ILClass_Name(implPrivate->classInfo));
+			#endif
+			if(implPrivate)
 			{
-				imtIndex = (implPrivate->imtBase + posn) % IL_IMT_SIZE;
-				vtableIndex = (ILImplPrivate_Table(impl))[posn];
-				if(vtableIndex != (ILUInt32)(ILUInt16)0xFFFF)
+				/* Allocate a base identifer for the interface if necessary.
+				   This will probably never be used because interfaces are
+				   typically laid out before their implementing classes, and
+				   the allocation above will catch us.  But let's be paranoid
+				   and allocate the base identifier here anyway, just in case */
+				size = (ILUInt32)(implPrivate->vtableSize);
+				if(!(implPrivate->imtBase))
 				{
-					if(!(classPrivate->imt[imtIndex]))
+					implPrivate->imtBase = process->imtBase;
+					process->imtBase += size;
+				}
+
+				/* Process the members of this interface */
+				for(posn = 0; posn < size; ++posn)
+				{
+					imtIndex = (implPrivate->imtBase + posn) % IL_IMT_SIZE;
+					vtableIndex = (ILImplPrivate_Table(impl))[posn];
+					if(vtableIndex != (ILUInt32)(ILUInt16)0xFFFF)
 					{
-						/* No conflict at this table position */
-						classPrivate->imt[imtIndex] =
-							classPrivate->vtable[vtableIndex];
-					}
-					else
-					{
-						/* We have encountered a conflict in the table */
-						classPrivate->imt[imtIndex] =
-							(ILMethod *)(ILNativeInt)(-1);
+						if(!(classPrivate->imt[imtIndex]))
+						{
+							/* No conflict at this table position */
+						#ifdef IL_USE_JIT
+							classPrivate->imt[imtIndex] =
+								classPrivate->jitVtable[vtableIndex];
+						#else
+							classPrivate->imt[imtIndex] =
+								classPrivate->vtable[vtableIndex];
+						#endif
+						}
+						else
+						{
+							/* We have encountered a conflict in the table */
+							classPrivate->imt[imtIndex] =
+							#ifdef IL_USE_JIT
+								(void *)(ILNativeInt)(-1);
+							#else
+								(ILMethod *)(ILNativeInt)(-1);
+							#endif
+						}
 					}
 				}
 			}
+			impl = impl->next;
 		}
-		impl = impl->next;
+		if(!parentPrivate->classInfo->parent)
+		{
+			break;
+		}
+		parentPrivate = (ILClassPrivate*) (ILClassGetParent(parentPrivate->classInfo)->userData);
 	}
 
 	/* Clear positions in the table that indicate conflicts */
 	for(posn = 0; posn < IL_IMT_SIZE; ++posn)
 	{
+	#ifdef IL_USE_JIT
+		if(classPrivate->imt[posn] == (void *)(ILNativeInt)(-1))
+	#else
 		if(classPrivate->imt[posn] == (ILMethod *)(ILNativeInt)(-1))
+	#endif
 		{
 			classPrivate->imt[posn] = 0;
 		}
+	#ifndef IL_USE_JIT
+		#if IL_DEBUG_IMTS
+		if(classPrivate->imt[posn] != 0)
+		{
+			fprintf(stderr, "\t %d : %s\n", posn, ILMethod_Name(classPrivate->imt[posn]));
+		}
+		#endif
+	#endif
 	}
+
+	#if IL_DEBUG_IMTS
+	fprintf(stderr, "}\n");
+	#endif
 }
 
 #endif /* IL_USE_IMTS */
@@ -546,7 +611,7 @@ static void BuildIMT(ILClass *info, ILClassPrivate *classPrivate)
  * Lay out a particular class.  Returns zero if there
  * is something wrong with the class definition.
  */
-static int LayoutClass(ILClass *info, LayoutInfo *layout)
+static int LayoutClass(ILExecProcess *process, ILClass *info, LayoutInfo *layout)
 {
 	ILClassLayout *classLayout;
 	ILFieldLayout *fieldLayout;
@@ -558,7 +623,7 @@ static int LayoutClass(ILClass *info, LayoutInfo *layout)
 	ILUInt32 explicitSize;
 	int allowFieldLayout;
 	int allowRVALayout;
-	ILClassPrivate *classPrivate;
+	ILClassPrivate *volatile classPrivate;
 	ILField *field;
 	ILMethod *method;
 	ILMethod *ancestor;
@@ -566,6 +631,9 @@ static int LayoutClass(ILClass *info, LayoutInfo *layout)
 	ILClass *parent;
 	ILImplements *implements;
 	ILMethodCode code;
+#ifdef IL_USE_JIT
+	void **jitVtable;
+#endif
 
 	/* Determine if we have already tried to lay out this class */
 	if(info->userData)
@@ -590,6 +658,10 @@ static int LayoutClass(ILClass *info, LayoutInfo *layout)
 			layout->hasFinalizer = classPrivate->hasFinalizer;
 			layout->managedInstance = classPrivate->managedInstance;
 			layout->managedStatic = classPrivate->managedStatic;
+		#ifdef IL_USE_JIT
+			layout->jitVtable = classPrivate->jitVtable;
+			layout->jitTypes = &(classPrivate->jitTypes);
+		#endif
 			return 1;
 		}
 	}
@@ -600,7 +672,7 @@ static int LayoutClass(ILClass *info, LayoutInfo *layout)
 		   garbage collector so that the object pointers that
 		   it contains will be visible to the collector */
 		classPrivate = (ILClassPrivate *)
-				(ILGCAllocPersistent(sizeof(ILClassPrivate)));
+				(ILGCAlloc(sizeof(ILClassPrivate)));
 		if(!classPrivate)
 		{
 			return 0;
@@ -609,6 +681,7 @@ static int LayoutClass(ILClass *info, LayoutInfo *layout)
 		info->userData = (void *)classPrivate;
 		classPrivate->inLayout = 1;
 		classPrivate->gcTypeDescriptor = IL_MAX_NATIVE_UINT;
+		classPrivate->process = process;
 	}
 
 	/* Lay out the parent class first */
@@ -616,7 +689,7 @@ static int LayoutClass(ILClass *info, LayoutInfo *layout)
 	{
 		/* Use "ILClassGetParent" to resolve cross-image links */
 		parent = ILClassGetParent(info);
-		if(!LayoutClass(parent, layout))
+		if(!LayoutClass(process, parent, layout))
 		{
 			info->userData = 0;
 			return 0;
@@ -638,12 +711,15 @@ static int LayoutClass(ILClass *info, LayoutInfo *layout)
 
 	/* Zero the static size, which must be recomputed for each class */
 	layout->staticSize = 0;
+#ifdef IL_USE_JIT
+	_ILJitTypesInit(&(classPrivate->jitTypes));
+#endif
 
 	/* Lay out the interfaces that this class implements */
 	implements = info->implements;
 	while(implements != 0)
 	{
-		if(!LayoutClass(ILClassResolve(implements->interface), &typeLayout))
+		if(!LayoutClass(process, ILClassResolve(implements->interface), &typeLayout))
 		{
 			info->userData = 0;
 			return 0;
@@ -698,7 +774,7 @@ static int LayoutClass(ILClass *info, LayoutInfo *layout)
 		if((field->member.attributes & IL_META_FIELDDEF_STATIC) == 0)
 		{
 			/* Get the layout information for this field's type */
-			if(!LayoutType(field->member.signature, &typeLayout))
+			if(!LayoutType(process, field->member.signature, &typeLayout))
 			{
 				info->userData = 0;
 				return 0;
@@ -804,6 +880,13 @@ static int LayoutClass(ILClass *info, LayoutInfo *layout)
 	classPrivate->alignment = layout->alignment;
 	classPrivate->nativeSize = layout->nativeSize;
 	classPrivate->nativeAlignment = layout->nativeAlignment;
+#ifdef IL_USE_JIT
+	if(!ILJitTypeCreate(classPrivate, process))
+	{
+		info->userData = 0;
+		return 0;
+	}
+#endif
 	classPrivate->inLayout = 0;
 
 	/* Allocate the static fields.  We must do this after the
@@ -827,7 +910,7 @@ static int LayoutClass(ILClass *info, LayoutInfo *layout)
 			if(!fieldRVA || !allowRVALayout)
 			{
 				/* Get the layout information for this field's type */
-				if(!LayoutType(field->member.signature, &typeLayout))
+				if(!LayoutType(process, field->member.signature, &typeLayout))
 				{
 					info->userData = 0;
 					return 0;
@@ -838,16 +921,9 @@ static int LayoutClass(ILClass *info, LayoutInfo *layout)
 				   function has acquired the metadata lock on the process */
 				if(ILFieldIsThreadStatic(field))
 				{
-					ILExecThread *thread = ILExecThreadCurrent();
-					if(!thread)
-					{
-						/* This shouldn't happen if the engine has
-						   been initialized correctly */
-						continue;
-					}
 					/* Store the slot number in the "offset" field and
 					   the field size in the "nativeOffset" field */
-					field->offset = (thread->process->numThreadStaticSlots)++;
+					field->offset = (process->numThreadStaticSlots)++;
 					field->nativeOffset = typeLayout.size;
 					continue;
 				}
@@ -915,6 +991,15 @@ static int LayoutClass(ILClass *info, LayoutInfo *layout)
 			{
 				/* Use the same index as the ancestor */
 				method->index = ancestor->index;
+			#ifdef IL_USE_JIT
+				if(!ILJitFunctionCreateFromAncestor(process->coder,
+													method,
+													ancestor))
+				{
+					info->userData = 0;
+					return 0;
+				}
+			#endif
 			}
 			else
 			{
@@ -937,6 +1022,15 @@ static int LayoutClass(ILClass *info, LayoutInfo *layout)
 		return 0;
 	}
 
+#ifdef IL_USE_JIT
+	if(!ILJitCreateFunctionsForClass(process->coder, info))
+	{
+		ILJitTypesDestroy(&(classPrivate->jitTypes));
+		info->userData = 0;
+		return 0;
+	}
+#endif
+
 	/* Allocate the vtable and copy the parent's vtable into it */
 	if((vtable = (ILMethod **)
 			ILMemStackAllocItem(&(info->programItem.image->memStack),
@@ -945,9 +1039,21 @@ static int LayoutClass(ILClass *info, LayoutInfo *layout)
 		info->userData = 0;
 		return 0;
 	}
+#ifdef IL_USE_JIT
+	if((jitVtable = (void **)
+			ILMemStackAllocItem(&(info->programItem.image->memStack),
+						        layout->vtableSize * sizeof(void *))) == 0)
+	{
+		info->userData = 0;
+		return 0;
+	}
+#endif
 	if(explicitSize > 0)
 	{
 		ILMemCpy(vtable, layout->vtable, explicitSize * sizeof(ILMethod *));
+	#ifdef IL_USE_JIT
+		ILMemCpy(jitVtable, layout->jitVtable, explicitSize * sizeof(ILMethod *));
+	#endif
 	}
 
 	/* Override the vtable slots with this class's method implementations */
@@ -958,6 +1064,9 @@ static int LayoutClass(ILClass *info, LayoutInfo *layout)
 		if((method->member.attributes & IL_META_METHODDEF_VIRTUAL) != 0)
 		{
 			vtable[method->index] = method;
+		#ifdef IL_USE_JIT
+			jitVtable[method->index] = jit_function_to_vtable_pointer(method->userData);
+		#endif
 		}
 	}
 
@@ -989,26 +1098,34 @@ static int LayoutClass(ILClass *info, LayoutInfo *layout)
 	classPrivate->managedInstance = layout->managedInstance;
 	classPrivate->managedStatic = layout->managedStatic;
 	layout->vtable = vtable;
+#ifdef IL_USE_JIT
+	classPrivate->jitVtable = jitVtable;
+	layout->jitVtable = jitVtable;
+#endif
 
 #ifdef IL_USE_IMTS
 	/* Build the interface method table for this class */
-	BuildIMT(info, classPrivate);
+	BuildIMT(process, info, classPrivate);
 #endif
 
 	/* Done */
+	/* add the classprivate data to the liked list */
+	classPrivate->nextClassPrivate = process->firstClassPrivate;
+	process->firstClassPrivate = classPrivate;
+
 	return 1;
 }
 
-int _ILLayoutClass(ILClass *info)
+int _ILLayoutClass(ILExecProcess *process, ILClass *info)
 {
 	LayoutInfo layout;
-	return LayoutClass(info, &layout);
+	return LayoutClass(process, info, &layout);
 }
 
-ILUInt32 _ILLayoutClassReturn(ILClass *info, ILUInt32 *alignment)
+ILUInt32 _ILLayoutClassReturn(ILExecProcess *process, ILClass *info, ILUInt32 *alignment)
 {
 	LayoutInfo layout;
-	if(LayoutClass(info, &layout))
+	if(LayoutClass(process, info, &layout))
 	{
 		*alignment = layout.alignment;
 		return layout.size;
@@ -1033,10 +1150,10 @@ int _ILLayoutAlreadyDone(ILClass *info)
 	}
 }
 
-ILUInt32 _ILSizeOfTypeLocked(ILType *type)
+ILUInt32 _ILSizeOfTypeLocked(ILExecProcess *process, ILType *type)
 {
 	LayoutInfo layout;
-	if(!LayoutType(type, &layout))
+	if(!LayoutType(process, type, &layout))
 	{
 		return 0;
 	}
@@ -1051,15 +1168,15 @@ ILUInt32 ILSizeOfType(ILExecThread *thread, ILType *type)
 	if(!ILType_IsValueType(type))
 	{
 		/* We can take a shortcut because the type is not a value type */
-		return _ILSizeOfTypeLocked(type);
+		return _ILSizeOfTypeLocked(_ILExecThreadProcess(thread), type);
 	}
 	else
 	{
 		/* We have to lock down the metadata first */
 		ILUInt32 size;
-		IL_METADATA_WRLOCK(thread);
-		size = _ILSizeOfTypeLocked(type);
-		IL_METADATA_UNLOCK(thread);
+		IL_METADATA_WRLOCK(_ILExecThreadProcess(thread));
+		size = _ILSizeOfTypeLocked(_ILExecThreadProcess(thread), type);
+		IL_METADATA_UNLOCK(_ILExecThreadProcess(thread));
 		return size;
 	}
 }
