@@ -2189,6 +2189,7 @@ void _IL_Array_Clear(ILExecThread *thread, ILObject *_array,
 	if(!array)
 	{
 		ILExecThreadThrowArgNull(thread, "array");
+		return;
 	}
 
 	/* Get the element type and size */
@@ -2395,8 +2396,8 @@ ILObject *_IL_Array_CreateArray_jiiii(ILExecThread *thread,
 
 	/* Create the array type and class structures */
 	elemType = ILClassToType((ILClass *)elementType);
-	type = ILTypeCreateArray(thread->process->context, (unsigned long)rank,
-							 elemType);
+	type = ILTypeFindOrCreateArray
+				(thread->process->context, (unsigned long)rank, elemType);
 	if(!type)
 	{
 		ILExecThreadThrowOutOfMemory(thread);
@@ -2409,6 +2410,7 @@ ILObject *_IL_Array_CreateArray_jiiii(ILExecThread *thread,
 		ILExecThreadThrowOutOfMemory(thread);
 		return 0;
 	}
+	classInfo = ILClassResolve(classInfo);
 
 	/* Compute the element size */
 	elemSize = ILSizeOfType(thread, elemType);
@@ -2612,13 +2614,33 @@ ILObject *_IL_Array_CreateArray_jaiai(ILExecThread *thread,
 	/* Create the array type and class structures */
 	rank = lengths->length;
 	elemType = ILClassToType((ILClass *)elementType);
-	type = ILTypeCreateArray(thread->process->context, (unsigned long)rank,
-							 elemType);
+	if(lowerBounds)
+	{
+		type = ILTypeCreateArray
+				(thread->process->context, (unsigned long)rank, elemType);
+	}
+	else
+	{
+		type = ILTypeFindOrCreateArray
+				(thread->process->context, (unsigned long)rank, elemType);
+	}
 	if(!type)
 	{
 		ILExecThreadThrowOutOfMemory(thread);
 		return 0;
 	}
+	if(lowerBounds)
+	{
+		for(dim = 0; dim < rank; ++dim)
+		{
+			ILTypeSetLowBound
+				(type, (unsigned long)(long)dim,
+				 (long)((ILInt32 *)ArrayToBuffer(lowerBounds))[dim]);
+			ILTypeSetSize
+				(type, (unsigned long)(long)dim,
+				 (long)((ILInt32 *)ArrayToBuffer(lengths))[dim]);
+		}
+	}
 	classInfo = ILClassFromType(ILProgramItem_Image(thread->method),
 								0, type, 0);
 	if(!classInfo)
@@ -2626,6 +2648,7 @@ ILObject *_IL_Array_CreateArray_jaiai(ILExecThread *thread,
 		ILExecThreadThrowOutOfMemory(thread);
 		return 0;
 	}
+	classInfo = ILClassResolve(classInfo);
 
 	/* Compute the element size */
 	elemSize = ILSizeOfType(thread, elemType);
@@ -2936,7 +2959,7 @@ ILObject *_IL_Array_Get_iii(ILExecThread *thread, ILObject *thisObj,
 										"Arg_InvalidArrayIndex");
 				return 0;
 			}
-			offset = index * marray->bounds[1].multiplier;
+			offset += index * marray->bounds[1].multiplier;
 			index = index3 - marray->bounds[2].lower;
 			if(((ILUInt32)index) >= ((ILUInt32)(marray->bounds[2].size)))
 			{
@@ -3008,6 +3031,7 @@ ILObject *_IL_Array_Get_ai(ILExecThread *thread, ILObject *thisObj,
 		}
 		offset += (ILUInt32)(index * marray->bounds[dim].multiplier);
 	}
+						
 	ptr = ((unsigned char *)(marray->data)) + offset * elemSize;
 
 	/* Box and return the array element */
@@ -3066,6 +3090,31 @@ ILObject *_IL_Array_GetRelative(ILExecThread *thread, ILObject *_this,
 	}
 }
 
+/* Helper function for all array stores */
+
+static int StoreObjectToArray(ILExecThread *thread, ILType * elemType,
+								ILObject *value, void* ptr,
+								ILObject **location)
+{
+	/* Copy the value into position at "ptr" */
+	if(ILType_IsPrimitive(elemType) || ILType_IsValueType(elemType))
+	{
+		if(ILExecThreadPromoteAndUnbox(thread, elemType, value, ptr))
+		{
+			return 1;
+		}
+	}
+	else if(ILTypeAssignCompatible
+					(ILProgramItem_Image(thread->method),
+				     (value ? ILClassToType(GetObjectClass(value)) : 0),
+				     elemType))
+	{
+		*(location) = value;
+		return 1;
+	}
+	return 0;
+}
+
 /*
  * private void Set(Object value, int index1, int index2, int index3);
  */
@@ -3152,7 +3201,7 @@ void _IL_Array_Set_Objectiii(ILExecThread *thread, ILObject *thisObj,
 										"Arg_InvalidArrayIndex");
 				return;
 			}
-			offset = index * marray->bounds[1].multiplier;
+			offset += index * marray->bounds[1].multiplier;
 			index = index3 - marray->bounds[2].lower;
 			if(((ILUInt32)index) >= ((ILUInt32)(marray->bounds[2].size)))
 			{
@@ -3173,28 +3222,12 @@ void _IL_Array_Set_Objectiii(ILExecThread *thread, ILObject *thisObj,
 		return;
 	}
 
-	/* Copy the value into position at "ptr" */
-	if(ILType_IsPrimitive(elemType) || ILType_IsValueType(elemType))
+	/* store to location and be done with it */
+	if(!StoreObjectToArray(thread, elemType, value, ptr, ptr))
 	{
-		if(!ILExecThreadUnbox(thread, elemType, value, ptr))
-		{
-			ILExecThreadThrowSystem
-					(thread, "System.ArgumentException",
-					 "Arg_ElementTypeMismatch");
-		}
-	}
-	else if(ILTypeAssignCompatible
-					(ILProgramItem_Image(thread->method),
-				     (value ? ILClassToType(GetObjectClass(value)) : 0),
-				     elemType))
-	{
-		*((ILObject **)ptr) = value;
-	}
-	else
-	{
-		ILExecThreadThrowSystem
-				(thread, "System.ArgumentException",
-				 "Arg_ElementTypeMismatch");
+		ILExecThreadThrowSystem(thread, "System.ArgumentException",
+					 		    "Arg_ElementTypeMismatch");
+		return;
 	}
 }
 
@@ -3244,25 +3277,12 @@ void _IL_Array_Set_Objectai(ILExecThread *thread, ILObject *thisObj,
 		offset += (ILUInt32)(index * marray->bounds[dim].multiplier);
 	}
 	ptr = ((unsigned char *)(marray->data)) + offset * elemSize;
-
-	/* Copy the value into position at "ptr" */
-	if(ILType_IsPrimitive(elemType) || ILType_IsValueType(elemType))
+	
+	if(StoreObjectToArray(thread, elemType, value, ptr, ptr))
 	{
-		if(!ILExecThreadUnbox(thread, elemType, value, ptr))
-		{
-			ILExecThreadThrowSystem
-					(thread, "System.ArgumentException",
-					 "Arg_ElementTypeMismatch");
-		}
+		return;
 	}
-	else if(ILTypeAssignCompatible
-					(ILProgramItem_Image(thread->method),
-				     (value ? ILClassToType(GetObjectClass(value)) : 0),
-				     elemType))
-	{
-		*((ILObject **)ptr) = value;
-	}
-	else
+	else	
 #endif /* IL_CONFIG_NON_VECTOR_ARRAYS */
 	{
 		ILExecThreadThrowSystem
@@ -3301,24 +3321,12 @@ void _IL_Array_SetRelative(ILExecThread *thread, ILObject *_this,
 	#endif
 	}
 
-	/* Copy the value into position in the array */
-	if(ILType_IsPrimitive(elemType) || ILType_IsValueType(elemType))
+	if(StoreObjectToArray(thread, elemType, value, 
+							((unsigned char *)buf) + index * size,
+							&(((ILObject **)buf)[index])))
 	{
-		if(!ILExecThreadUnbox(thread, elemType, value,
-							  ((unsigned char *)buf) + index * size))
-		{
-			ILExecThreadThrowSystem
-					(thread, "System.ArgumentException",
-					 "Arg_ElementTypeMismatch");
-		}
-	}
-	else if(ILTypeAssignCompatible
-					(ILProgramItem_Image(thread->method),
-				     (value ? ILClassToType(GetObjectClass(value)) : 0),
-				     elemType))
-	{
-		((ILObject **)buf)[index] = value;
-	}
+		return;
+	}		
 	else
 	{
 		ILExecThreadThrowSystem
@@ -3335,7 +3343,7 @@ void _IL_Array_SetRelative(ILExecThread *thread, ILObject *_this,
 void _IL_Buffer_Copy(ILExecThread *thread,
 					 ILObject *_src, ILInt32 srcOffset,
 					 ILObject *_dst, ILInt32 dstOffset,
-					 int count)
+					 ILInt32 count)
 {
 	System_Array *src = (System_Array *)_src;
 	System_Array *dst = (System_Array *)_dst;
