@@ -461,15 +461,24 @@ ILBool _IL_Monitor_InternalWait(ILExecThread *thread,
 
 		return 0;
 
-	case 1:
-	case IL_WAIT_TIMEOUT:
 	case IL_WAIT_FAILED:
+	case 1:
 
-		/* Success or timed out */
+		/* Success or wait failed */
 
 		/* Returning 1 because the lock is always regained */
+
 		return 1;
 
+	case IL_WAIT_TIMEOUT:
+
+		/* Return 0 to indicate that the thread competed and reacquired 
+		   the lock after it was woken and that it was woken because it 
+		   timed out waiting for a pulse rather than because it received 
+		   a pulse. */
+
+		return 0;
+
 	default:
 
 		/* Handle ThreadAbort etc */
