@@ -28,7 +28,8 @@ extern	"C" {
 /*
  * Stubs for coder functions.
  */
-static ILCoder *Coder_Create(ILUInt32 size, unsigned long cachePageSize)
+static ILCoder *Coder_Create(ILExecProcess *process, ILUInt32 size,
+							 unsigned long cachePageSize)
 {
 	return 0;
 }
