@@ -88,11 +88,11 @@ static ffi_type ffi_type_typedref =
 /*
  * Forward declarations.
  */
-static ffi_type *TypeToFFI(ILType *type, int isInternal);
+static ffi_type *TypeToFFI(ILExecProcess *process, ILType *type, int isInternal);
 
 #ifdef IL_CONFIG_PINVOKE
 
-static ffi_type *StructToFFI(ILClass *classInfo);
+static ffi_type *StructToFFI(ILExecProcess *process, ILClass *classInfo);
 
 #if !FFI_NO_STRUCTS
 
@@ -101,8 +101,8 @@ static ffi_type *StructToFFI(ILClass *classInfo);
  * about the non-static fields of a class.  Returns zero if
  * out of memory.
  */
-static int PopulateStructFFI(ILClass *classInfo, ffi_type **fieldTypes,
-							 unsigned *posn)
+static int PopulateStructFFI(ILExecProcess *process, ILClass *classInfo,
+							  ffi_type **fieldTypes, unsigned *posn)
 {
 	ILClass *parent;
 	ILField *field;
@@ -113,7 +113,7 @@ static int PopulateStructFFI(ILClass *classInfo, ffi_type **fieldTypes,
 	parent = ILClassGetParent(classInfo);
 	if(parent)
 	{
-		if(!PopulateStructFFI(parent, fieldTypes, posn))
+		if(!PopulateStructFFI(process, parent, fieldTypes, posn))
 		{
 			return 0;
 		}
@@ -130,7 +130,7 @@ static int PopulateStructFFI(ILClass *classInfo, ffi_type **fieldTypes,
 			if(ILType_IsValueType(type))
 			{
 				/* Process an embedded structure type */
-				ffi = StructToFFI(ILType_ToValueType(type));
+				ffi = StructToFFI(process, ILType_ToValueType(type));
 				if(!ffi)
 				{
 					return 0;
@@ -139,7 +139,7 @@ static int PopulateStructFFI(ILClass *classInfo, ffi_type **fieldTypes,
 			else
 			{
 				/* Process a non-structure type */
-				ffi = TypeToFFI(type, 0);
+				ffi = TypeToFFI(process, type, 0);
 			}
 			fieldTypes[(*posn)++] = ffi;
 		}
@@ -155,7 +155,7 @@ static int PopulateStructFFI(ILClass *classInfo, ffi_type **fieldTypes,
  * Convert a "struct" class into a "ffi" type descriptor.
  * Returns zero if out of memory.
  */
-static ffi_type *StructToFFI(ILClass *classInfo)
+static ffi_type *StructToFFI(ILExecProcess *process, ILClass *classInfo)
 {
 #if !FFI_NO_STRUCTS
 	ILClass *current;
@@ -190,7 +190,7 @@ static ffi_type *StructToFFI(ILClass *classInfo)
 	else
 	{
 		/* Use the explicit layout information from "layout.c" */
-		explicitSize = _ILLayoutClassReturn(classInfo, &explicitAlignment);
+		explicitSize = _ILLayoutClassReturn(process, classInfo, &explicitAlignment);
 	}
 
 	/* Allocate space for the struct's type descriptor */
@@ -213,7 +213,7 @@ static ffi_type *StructToFFI(ILClass *classInfo)
 	if(!ILClass_IsExplicitLayout(classInfo))
 	{
 		numFields = 0;
-		if(!PopulateStructFFI(classInfo, fieldTypes, &numFields))
+		if(!PopulateStructFFI(process, classInfo, fieldTypes, &numFields))
 		{
 			ILFree(descr);
 			return 0;
@@ -239,7 +239,7 @@ static ffi_type *StructToFFI(ILClass *classInfo)
 /*
  * Convert an IL type into an "ffi" type descriptor.
  */
-static ffi_type *TypeToFFI(ILType *type, int isInternal)
+static ffi_type *TypeToFFI(ILExecProcess *process, ILType *type, int isInternal)
 {
 	if(ILType_IsPrimitive(type))
 	{
@@ -279,7 +279,7 @@ static ffi_type *TypeToFFI(ILType *type, int isInternal)
 	else if(!isInternal && ILType_IsValueType(type))
 	{
 		/* Structure that is passed by value to a PInvoke method */
-		ffi_type *ffi = StructToFFI(ILClassResolve(ILType_ToValueType(type)));
+		ffi_type *ffi = StructToFFI(process, ILClassResolve(ILType_ToValueType(type)));
 		return (ffi ? ffi : &ffi_type_pointer);
 	}
 #endif
@@ -290,7 +290,7 @@ static ffi_type *TypeToFFI(ILType *type, int isInternal)
 	}
 }
 
-void *_ILMakeCifForMethod(ILMethod *method, int isInternal)
+void *_ILMakeCifForMethod(ILExecProcess *process, ILMethod *method, int isInternal)
 {
 	ILType *signature = ILMethod_Signature(method);
 	ILType *returnType = ILTypeGetEnumType(ILTypeGetReturn(signature));
@@ -333,7 +333,7 @@ void *_ILMakeCifForMethod(ILMethod *method, int isInternal)
 	args = ((ffi_type **)(cif + 1));
 
 	/* Convert the return type */
-	rtype = TypeToFFI(modReturnType, isInternal);
+	rtype = TypeToFFI(process, modReturnType, isInternal);
 
 	/* Convert the argument types */
 	arg = 0;
@@ -354,7 +354,7 @@ void *_ILMakeCifForMethod(ILMethod *method, int isInternal)
 	}
 	for(param = 1; param <= numParams; ++param)
 	{
-		args[arg++] = TypeToFFI(ILTypeGetEnumType
+		args[arg++] = TypeToFFI(process, ILTypeGetEnumType
 									(ILTypeGetParam(signature, param)),
 							    isInternal);
 	}
@@ -378,7 +378,7 @@ void *_ILMakeCifForMethod(ILMethod *method, int isInternal)
 	return (void *)cif;
 }
 
-void *_ILMakeCifForConstructor(ILMethod *method, int isInternal)
+void *_ILMakeCifForConstructor(ILExecProcess *process, ILMethod *method, int isInternal)
 {
 	ILType *signature = ILMethod_Signature(method);
 	ILUInt32 numArgs;
@@ -420,7 +420,7 @@ void *_ILMakeCifForConstructor(ILMethod *method, int isInternal)
 	}
 	for(param = 1; param <= numParams; ++param)
 	{
-		args[arg++] = TypeToFFI(ILTypeGetEnumType
+		args[arg++] = TypeToFFI(process, ILTypeGetEnumType
 									(ILTypeGetParam(signature, param)),
 							    isInternal);
 	}
@@ -942,46 +942,59 @@ static void UnpackDelegateResult(ILExecThread *thread, ILMethod *method,
 	}
 }
 
+typedef struct {
+	ILExecThread *thread;
+	ffi_cif *cif;
+	void *result;
+	void **args;
+	System_Delegate *delegate;
+} ILDelegateInvokeParams;
+
 /*
- * Invoke a delegate from a closure.
+ * Do the real delegate invokation.
  */
-static void DelegateInvoke(ffi_cif *cif, void *result,
-						   void **args, void *delegate)
+static void _DelegateInvoke(ILDelegateInvokeParams *params)
 {
-	ILExecThread *thread = ILExecThreadCurrent();
 	ILMethod *method;
 	ILType *type;
 	ILUInt32 size;
 	PackDelegateUserData userData;
 
 	/* If this is a multicast delegate, then execute "prev" first */
-	if(((System_Delegate *)delegate)->prev)
+	if(params->delegate->prev)
 	{
-		DelegateInvoke(cif, result, args,
-					   ((System_Delegate *)delegate)->prev);
-		if(ILExecThreadHasException(thread))
+		ILDelegateInvokeParams prevParams;
+
+		prevParams.thread = params->thread;
+		prevParams.cif = params->cif;
+		prevParams.result = params->result;
+		prevParams.args = params->args;
+		prevParams.delegate = (System_Delegate *)(params->delegate->prev);
+;
+		_DelegateInvoke(&prevParams);
+		if(ILExecThreadHasException(params->thread))
 		{
 			return;
 		}
 	}
 
 	/* Extract the method from the delegate */
-	method = ((System_Delegate *)delegate)->methodInfo;
+	method = params->delegate->methodInfo;
 	if(!method)
 	{
-		ILExecThreadThrowSystem(thread, "System.MissingMethodException",
+		ILExecThreadThrowSystem(params->thread, "System.MissingMethodException",
 								(const char *)0);
 		return;
 	}
 
 	/* Call the method */
-	userData.args = args;
+	userData.args = params->args;
 	userData.pinvokeInfo = (ILMethod *)ILTypeGetDelegateMethod
-		(ILType_FromClass(GetObjectClass(delegate)));
+		(ILType_FromClass(GetObjectClass(params->delegate)));
 	userData.needThis = 0;
-	if(_ILCallMethod(thread, method,
-				     UnpackDelegateResult, result,
-				     0, ((System_Delegate *)delegate)->target,
+	if(_ILCallMethod(params->thread, method,
+				     UnpackDelegateResult, params->result,
+				     0, params->delegate->target,
 				     PackDelegateParams, &userData))
 	{
 		/* An exception occurred, which is already stored in the thread */
@@ -991,8 +1004,70 @@ static void DelegateInvoke(ffi_cif *cif, void *result,
 		{
 			/* Clear the native return value, because we cannot assume
 			   that the native caller knows how to handle exceptions */
-			size = ILSizeOfType(thread, type);
-			ILMemZero(result, size);
+			size = ILSizeOfType(params->thread, type);
+			ILMemZero(params->result, size);
+		}
+	}
+}
+
+/*
+ * Invoke the delegate from a new thread.
+ */
+static void *_DelegateInvokeFromNewThread(void *params)
+{
+	ILThread *thread = ILThreadSelf();
+	ILClassPrivate *classPrivate = GetObjectClassPrivate(((ILDelegateInvokeParams *)params)->delegate);
+	ILExecProcess *process = classPrivate->process;
+
+	if((((ILDelegateInvokeParams *)params)->thread = ILThreadRegisterForManagedExecution(process, thread)))
+	{
+		_DelegateInvoke((ILDelegateInvokeParams *)params);
+		ILThreadUnregisterForManagedExecution(thread);
+	}
+	return 0;
+}
+
+/*
+ * Invoke a delegate from a closure.
+ */
+static void DelegateInvoke(ffi_cif *cif, void *result,
+						   void **args, void *delegate)
+{
+	ILThread *thread = ILThreadSelf();
+	ILClassPrivate *classPrivate = GetObjectClassPrivate(delegate);
+	ILExecProcess *process = classPrivate->process;
+	ILDelegateInvokeParams params;
+
+	params.thread = 0;
+	params.cif = cif;
+	params.result = result;
+	params.args = args;
+	params.delegate = (System_Delegate *)delegate;
+	if(!thread)
+	{
+		/* callback was invoked by a non pnet thread. */
+		
+		ILThreadRunSelf(_DelegateInvokeFromNewThread, (void *)&params);
+	}
+	else
+	{
+		params.thread = _ILExecThreadFromThread(thread);
+
+		if(params.thread)
+		{
+			IL_BEGIN_EXECPROCESS_SWITCH(params.thread, process)
+			_DelegateInvoke(&params);
+			IL_END_EXECPROCESS_SWITCH(params.thread)
+		}
+		else
+		{
+			/* thread is not registerd for managed execution */
+			if((params.thread = ILThreadRegisterForManagedExecution(process, thread)))
+			{
+
+				_DelegateInvoke(&params);
+				ILThreadUnregisterForManagedExecution(thread);
+			}
 		}
 	}
 }
@@ -1032,7 +1107,7 @@ static void MethodInvoke(ffi_cif *cif, void *result,
 
 #endif /* FFI_CLOSURES */
 
-void *_ILMakeClosureForDelegate(ILObject *delegate, ILMethod *method)
+void *_ILMakeClosureForDelegate(ILExecProcess *process, ILObject *delegate, ILMethod *method)
 {
 #if FFI_CLOSURES
 	ILType *signature = ILMethod_Signature(method);
@@ -1058,7 +1133,7 @@ void *_ILMakeClosureForDelegate(ILObject *delegate, ILMethod *method)
 	args = ((ffi_type **)(cif + 1));
 
 	/* Convert the return type */
-	rtype = TypeToFFI(returnType, 0);
+	rtype = TypeToFFI(process, returnType, 0);
 
 	/* Convert the argument types */
 	arg = 0;
@@ -1069,7 +1144,7 @@ void *_ILMakeClosureForDelegate(ILObject *delegate, ILMethod *method)
 	}
 	for(param = 1; param <= numArgs; ++param)
 	{
-		args[arg++] = TypeToFFI(ILTypeGetEnumType
+		args[arg++] = TypeToFFI(process, ILTypeGetEnumType
 									(ILTypeGetParam(signature, param)), 0);
 	}
 
@@ -1116,6 +1191,7 @@ void *_ILMakeClosureForDelegate(ILObject *delegate, ILMethod *method)
 	return (void *)closure;
 #else	/* !FFI_CLOSURES */
 	/* libffi does not support closures */
+	fprintf(stderr, "libffi does not support closures on this arch.\n");
 	return 0;
 #endif	/* !FFI_CLOSURES */
 }
