@@ -135,6 +135,7 @@ ILExecProcess *ILExecProcessCreate(unsigned long stackSize, unsigned long cacheP
 	process->gcHandles = 0;
 	process->entryImage = 0;
 	process->internalClassTable = 0;
+	process->firstClassPrivate = 0;
 #ifdef IL_CONFIG_DEBUG_LINES
 	process->debugHookFunc = 0;
 	process->debugHookData = 0;
@@ -168,8 +169,13 @@ ILExecProcess *ILExecProcessCreate(unsigned long stackSize, unsigned long cacheP
 	/* Associate the process with the context */
 	ILContextSetUserData(process->context, process);
 
+#ifdef IL_USE_JIT
+	/* Initialize the JIT coder */
+	process->coder = ILCoderCreate(&_ILJITCoderClass, process, 100000, cachePageSize);
+#else
 	/* Initialize the CVM coder */
-	process->coder = ILCoderCreate(&_ILCVMCoderClass, 100000, cachePageSize);
+	process->coder = ILCoderCreate(&_ILCVMCoderClass, process, 100000, cachePageSize);
+#endif
 	if(!(process->coder))
 	{
 		ILExecProcessDestroy(process);
