@@ -48,6 +48,14 @@ void ILExecThreadSetException(ILExecThread *thread, ILObject *obj)
 
 void _ILSetExceptionStackTrace(ILExecThread *thread, ILObject *object);
 
+void ILExecThreadThrow(ILExecThread *thread, ILObject *object)
+{
+	if(!ILExecThreadHasException(thread))
+	{
+		ILExecThreadSetException(thread, object);
+	}
+}
+
 void ILExecThreadThrowSystem(ILExecThread *thread, const char *typeName,
 							 const char *resourceName)
 {
@@ -55,7 +63,7 @@ void ILExecThreadThrowSystem(ILExecThread *thread, const char *typeName,
 	ILString *resourceString = 0;
 	ILObject *object;
 
-	/* Bail out if there already is a pending exception */
+	/* Bail out if there already is a pending exception or if the thread is aborting */
 	if(ILExecThreadHasException(thread))
 	{
 		return;
@@ -108,7 +116,7 @@ void ILExecThreadThrowArgRange(ILExecThread *thread, const char *paramName,
 	ILString *paramString;
 	ILObject *object;
 
-	/* Bail out if there already is a pending exception */
+	/* Bail out if there already is a pending exception or if the thread is aborting */
 	if(ILExecThreadHasException(thread))
 	{
 		return;
@@ -169,7 +177,7 @@ void ILExecThreadThrowArgNull(ILExecThread *thread, const char *paramName)
 	ILString *paramString;
 	ILObject *object;
 
-	/* Bail out if there already is a pending exception */
+	/* Bail out if there already is a pending exception or if the thread is aborting */
 	if(ILExecThreadHasException(thread))
 	{
 		return;
@@ -203,24 +211,22 @@ void ILExecThreadThrowOutOfMemory(ILExecThread *thread)
 	}
 }
 
-void _ILExecThreadThrowThreadAbortException(ILExecThread *thread, ILObject *stateInfo)
+ILObject *_ILExecThreadNewThreadAbortException(ILExecThread *thread, ILObject *stateInfo)
 {
 	ILObject *object;
 
 	/* Bail out if there already is a pending exception */
-	if(ILExecThreadHasException(thread))
-	{
-		return;
-	}
-
 	object = ILExecThreadNew(thread, "System.Threading.ThreadAbortException",
 						"(ToSystem.Object;)V", stateInfo);
 	
 	_ILSetExceptionStackTrace(thread, object);
-	if(!ILExecThreadHasException(thread))
-	{
-		ILExecThreadSetException(thread, object);
-	}
+	
+	return object;
+}
+
+int ILExecThreadIsThreadAbortException(ILExecThread *thread, ILObject *object)
+{
+	return GetObjectClass(object) == thread->process->threadAbortClass;
 }
 
 void ILExecThreadPrintException(ILExecThread *thread)
@@ -252,15 +258,6 @@ void ILExecThreadPrintException(ILExecThread *thread)
 		exception = thread->process->outOfMemoryObject;
 	}
 
-	/*
-	 *	Don't print out ThreadAbortExceptions.
-	 */
-
-	if (GetObjectClass(exception) == thread->process->threadAbortClass)
-	{
-		return;
-	}
-
 	/* Attempt to use "ToString" to format the exception, but not
 	   if we know that the exception is reporting out of memory */
 	if(exception != thread->process->outOfMemoryObject)		
