@@ -37,6 +37,8 @@ extern	"C" {
  */
 #if defined(CVM_X86)
 	#include "md_x86.h"
+#elif defined(CVM_X86_64)
+	#include "md_amd64.h"
 #elif defined(CVM_ARM)
 	#include "md_arm.h"
 #elif defined(CVM_PPC)
@@ -983,7 +985,7 @@ static void GetTopTwoFPRegisters(MDUnroll *unroll, int *reg1,
 		if(MD_IS_FREG(unroll->pseudoStack[unroll->pseudoStackSize - 2]) &&
 		   MD_IS_FREG(unroll->pseudoStack[unroll->pseudoStackSize - 1]))
 		{
-#ifdef CVM_X86
+#if defined(CVM_X86) || defined(CVM_X86_64)
 			if(fprem)
 			{
 				md_freg_swap(unroll->out);
@@ -1009,7 +1011,7 @@ static void GetTopTwoFPRegisters(MDUnroll *unroll, int *reg1,
 				(unroll->out, MD_FREG_0, MD_REG_STACK, unroll->stackHeight);
 
 			/* Put the values into the correct order */
-#ifdef CVM_X86
+#if defined(CVM_X86) || defined(CVM_X86_64)
 			if(!fprem)
 #endif
 			{
@@ -1031,7 +1033,7 @@ static void GetTopTwoFPRegisters(MDUnroll *unroll, int *reg1,
 
 	/* Load the top of the CVM stack onto the FP stack */
 	unroll->stackHeight -= CVM_WORDS_PER_NATIVE_FLOAT * sizeof(CVMWord) * 2;
-#ifdef CVM_X86
+#if defined(CVM_X86) || defined(CVM_X86_64)
 	if(fprem)
 	{
 		md_load_membase_float_native(unroll->out, MD_FREG_0, MD_REG_STACK,
@@ -1178,8 +1180,8 @@ static void GetFPAndWordRegisters(MDUnroll *unroll, int *reg1, int *reg2)
 	/* If we have 1 word value on the stack, then load the FP into MD_FREG_0 */
 	if(unroll->pseudoStackSize == 1 && !MD_IS_FREG(unroll->pseudoStack[0]))
 	{
-		*reg1 = unroll->pseudoStack[0];
-		*reg2 = MD_FREG_0;
+		*reg1 = MD_FREG_0;
+		*reg2 = unroll->pseudoStack[0] & ~MD_NATIVE_REG_MASK;
 		unroll->pseudoStack[0] = *reg1;
 		unroll->pseudoStack[1] = *reg2 | MD_NATIVE_REG_MASK;
 		unroll->pseudoStackSize = 2;
