@@ -39,7 +39,7 @@
 #define V_INLINE_FIELD_STORE (2)
 #define V_INLINE_CONST_LOAD  (3)
 
-static int TryInlineLoad(ILMethod *method, int numParams, 
+static int TryInlineLoad(ILExecProcess *process, ILMethod *method, int numParams, 
 							   ILMethodCode *code, unsigned *inlineOpcode, 
 							   unsigned char **inlinePc, ILField **field)
 {
@@ -133,7 +133,7 @@ static int TryInlineLoad(ILMethod *method, int numParams,
 	
 		retval = V_INLINE_FIELD_LOAD;
 		
-		*field = GetFieldToken(method, pc);
+		*field = GetFieldToken(process, method, pc);
 
 		if (*field == 0)
 		{
@@ -206,7 +206,7 @@ static int TryInlineLoad(ILMethod *method, int numParams,
 	return retval;
 }
 
-static int TryInlineStore(ILMethod *method, int numParams, 
+static int TryInlineStore(ILExecProcess *process, ILMethod *method, int numParams, 
 							   ILMethodCode *code, unsigned int *inlineOpcode, 
 							   unsigned char **inlinePc, ILField **field)
 {
@@ -296,7 +296,7 @@ static int TryInlineStore(ILMethod *method, int numParams,
 	*inlinePc = pc;
 	*inlineOpcode = opcode;
 
-	*field = GetFieldToken(method, pc);
+	*field = GetFieldToken(process, method, pc);
 
 	if (*field == 0)
 	{
@@ -329,7 +329,7 @@ static int TryInlineStore(ILMethod *method, int numParams,
 	return V_INLINE_FIELD_STORE;
 }
 
-static int TryInlineMethod(ILMethod *method, 
+static int TryInlineMethod(ILExecProcess *process, ILMethod *method, 
 						   int numParams, unsigned int *inlineOpcode, 
 						   unsigned char **inlinePc, ILField **field)
 {
@@ -346,12 +346,12 @@ static int TryInlineMethod(ILMethod *method,
 		return 0;
 	}
 
-	if ((retval = TryInlineLoad(method, numParams, &code, inlineOpcode, inlinePc, field)))
+	if ((retval = TryInlineLoad(process, method, numParams, &code, inlineOpcode, inlinePc, field)))
 	{
 		return retval;
 	}
 
-	if ((retval = TryInlineStore(method, numParams, &code, inlineOpcode, inlinePc, field)))
+	if ((retval = TryInlineStore(process, method, numParams, &code, inlineOpcode, inlinePc, field)))
 	{
 		return retval;
 	}
@@ -364,8 +364,8 @@ static int TryInlineMethod(ILMethod *method,
  * is not NULL, then write the call site signature to it.  If it is
  * NULL, then vararg method calls are disallowed.
  */
-static ILMethod *GetMethodToken(ILMethod *method, unsigned char *pc,
-								ILType **callSiteSig)
+static ILMethod *GetMethodToken(ILExecProcess *process, ILMethod *method,
+								unsigned char *pc, ILType **callSiteSig)
 {
 	ILUInt32 token;
 	ILMethod *methodInfo;
@@ -390,7 +390,7 @@ static ILMethod *GetMethodToken(ILMethod *method, unsigned char *pc,
 	methodInfo = (ILMethod *)ILMemberResolve((ILMember *)methodInfo);
 
 	/* Make sure that the method's class has been laid out */
-	if(!_ILLayoutClass(ILMethod_Owner(methodInfo)))
+	if(!_ILLayoutClass(process, ILMethod_Owner(methodInfo)))
 	{
 		return 0;
 	}
@@ -416,10 +416,10 @@ static ILMethod *GetMethodToken(ILMethod *method, unsigned char *pc,
  * is not NULL, then write the call site signature to it.  If it is
  * NULL, then vararg method calls are disallowed.
  */
-static ILMethod *GetConstructorToken(ILMethod *method, unsigned char *pc,
-									 ILType **callSiteSig)
+static ILMethod *GetConstructorToken(ILExecProcess *process, ILMethod *method,
+									 unsigned char *pc,ILType **callSiteSig)
 {
-	ILMethod *methodInfo = GetMethodToken(method, pc, callSiteSig);
+	ILMethod *methodInfo = GetMethodToken(process, method, pc, callSiteSig);
 
 	/* Bail out if no method found */
 	if(!methodInfo)
@@ -1382,7 +1382,7 @@ case IL_OP_JMP:
 	/* Jump to another method with exactly the same set of arguments */
 	if(unsafeAllowed)
 	{
-		methodInfo = GetMethodToken(method, pc, (ILType **)0);
+		methodInfo = GetMethodToken(_ILExecThreadProcess(thread), method, pc, (ILType **)0);
 		if(methodInfo && !ILMethod_IsAbstract(methodInfo))
 		{
 			if(ILMemberAccessible((ILMember *)methodInfo,
@@ -1419,7 +1419,8 @@ break;
 case IL_OP_CALL:
 {
 	/* Call a particular method directly */
-	methodInfo = GetMethodToken(method, pc, &methodSignature);
+	methodInfo = GetMethodToken(_ILExecThreadProcess(thread), method, pc,
+								&methodSignature);
 	if(methodInfo && !ILMethod_IsAbstract(methodInfo))
 	{
 		if(ILMemberAccessible((ILMember *)methodInfo, ILMethod_Owner(method)))
@@ -1467,8 +1468,8 @@ callNonvirtualFromVirtual:
 #endif
 				 		/* Inlineable method code generated by ILCoderCallInlineable */
 				 	}
-				 	else if ((tryInlineType = TryInlineMethod(methodInfo, numParams,
-										&tryInlineOpcode, &tryInlinePc, &fieldInfo)) != 0)
+				 	else if ((tryInlineType = TryInlineMethod(_ILExecThreadProcess(thread), methodInfo,
+										numParams, &tryInlineOpcode, &tryInlinePc, &fieldInfo)) != 0)
 					{
 #if !defined(IL_CONFIG_REDUCE_CODE) && !defined(IL_WITHOUT_TOOLS)
 						if (coderFlags & IL_CODER_FLAG_STATS)
@@ -1687,11 +1688,11 @@ callNonvirtualFromVirtual:
 								classType = ILField_Type(fieldInfo);
 								if(STK_UNARY == ILEngineType_O)
 								{
-									/* Accessing a field within an object reference */
-									if(IsSubClass(stack[stackSize - 1].typeInfo,
-												ILField_Owner(fieldInfo)))
+									if(!ILField_IsStatic(fieldInfo))
 									{
-										if(!ILField_IsStatic(fieldInfo))
+										/* Accessing a field within an object reference */
+										if(IsSubClass(stack[stackSize - 1].typeInfo,
+												ILField_Owner(fieldInfo)))
 										{
 											ILCoderLoadField(coder, ILEngineType_O,
 															stack[stackSize - 1].typeInfo,
@@ -1699,13 +1700,13 @@ callNonvirtualFromVirtual:
 										}
 										else
 										{
-											ILCoderPop(coder, ILEngineType_O, ILType_Invalid);
-											ILCoderLoadStaticField(coder, fieldInfo, classType);
+											VERIFY_TYPE_ERROR();
 										}
 									}
 									else
 									{
-										VERIFY_TYPE_ERROR();
+										ILCoderPop(coder, ILEngineType_O, ILType_Invalid);
+										ILCoderLoadStaticField(coder, fieldInfo, classType);
 									}
 								}
 								else if(!unsafeAllowed &&
@@ -1975,7 +1976,7 @@ break;
 case IL_OP_CALLVIRT:
 {
 	/* Call a virtual or interface method */
-	methodInfo = GetMethodToken(method, pc, &methodSignature);
+	methodInfo = GetMethodToken(_ILExecThreadProcess(thread), method, pc, &methodSignature);
 	if(methodInfo)
 	{
 		classInfo = ILMethod_Owner(method);
@@ -2046,7 +2047,8 @@ break;
 case IL_OP_NEWOBJ:
 {
 	/* Create a new object and call its constructor */
-	methodInfo = GetConstructorToken(method, pc, &methodSignature);
+	methodInfo = GetConstructorToken(_ILExecThreadProcess(thread), method, pc,
+									 &methodSignature);
 	if(methodInfo)
 	{
 		/* The construction sequence is different for objects and values */
@@ -2149,7 +2151,8 @@ break;
 case IL_OP_PREFIX + IL_PREFIX_OP_LDFTN:
 {
 	/* Load the address of a function onto the stack as "I" */
-	methodInfo = GetMethodToken(method, pc, (ILType **)0);
+	methodInfo = GetMethodToken(_ILExecThreadProcess(thread), method, pc,
+								(ILType **)0);
 	if(methodInfo && !ILMethod_IsAbstract(methodInfo))
 	{
 		if(ILMemberAccessible((ILMember *)methodInfo, ILMethod_Owner(method)))
@@ -2179,7 +2182,8 @@ break;
 case IL_OP_PREFIX + IL_PREFIX_OP_LDVIRTFTN:
 {
 	/* Load the address of a virtual function onto the stack as "I" */
-	methodInfo = GetMethodToken(method, pc, (ILType **)0);
+	methodInfo = GetMethodToken(_ILExecThreadProcess(thread), method, pc,
+								(ILType **)0);
 	if(methodInfo && STK_UNARY == ILEngineType_O)
 	{
 		classInfo = ILMethod_Owner(methodInfo);
