@@ -946,7 +946,7 @@ int ILLinkerMain(int argc, char *argv[])
 		{
 			if(!ILLinkerSetEntryPoint(linker, entryPoint))
 			{
-				fprintf(stderr, "%s: could not locate the entry point `%s'",
+				fprintf(stderr, "%s: could not locate the entry point `%s'\n",
 						outputFile, entryPoint);
 				errors = 1;
 			}
