@@ -418,6 +418,9 @@ struct _tagILWriter
 	unsigned long	backpatchLen;	/* Length of back-patch cache */
 	unsigned char  *backpatchBuf;	/* Buffer containing back-patch data */
 
+	/* Version string for the runtime header (e.g. "v1.1.4322") */
+	unsigned char	versionString[12];
+
 };
 
 #endif /* IL_USE_WRITER */
@@ -854,6 +857,9 @@ void WriteJavaClass(ILWriter *writer, ILClass *class);
 #define	IL_OFFSET_METHODSPEC_INST			2
 #define	IL_OFFSET_METHODSPEC_INST_LEN		3
 
+#define	IL_OFFSET_GENERICCON_PARAM			0
+#define	IL_OFFSET_GENERICCON_CONSTRAINT		1
+
 /*
  * Get the raw data associated with a particular token.
  * Returns zero if the token does not exist.  The "values"
