@@ -352,7 +352,9 @@ struct _tagILMethod
 	void           *userData;			/* User data for the runtime engine */
 	ILUInt32		index;				/* Data added by the runtime engine */
 	ILUInt32		count;				/* Profile count for the engine */
-
+#ifdef ENHANCED_PROFILER
+	ILUInt32		time;				/* Profile time counter for the engine */
+#endif
 };
 
 /*
