@@ -78,7 +78,6 @@ typedef struct
 struct _tagILCoder
 {
 	const ILCoderClass *classInfo;
-
 };
 
 /*
@@ -169,7 +168,8 @@ struct _tagILCoderClass
 	/*
 	 * Create a coder instance.  Returns NULL if not possible.
 	 */
-	ILCoder *(*create)(ILUInt32 size,unsigned long cachePageSize);
+	ILCoder *(*create)(ILExecProcess *process, ILUInt32 size,
+						unsigned long cachePageSize);
 	
 	/*
 	 * Enable debug mode.  The coder will output extra information
@@ -818,8 +818,8 @@ struct _tagILCoderClass
 /*
  * Helper macros for calling coder methods.
  */
-#define	ILCoderCreate(classInfo,size,cachePageSize)	\
-			((*((classInfo)->create))((size),(cachePageSize)))
+#define	ILCoderCreate(classInfo,process,size,cachePageSize)	\
+			((*((classInfo)->create))((process), (size), (cachePageSize)))
 #define	ILCoderEnableDebug(coder)	\
 			((*((coder)->classInfo->enableDebug))((coder)))
 #define	ILCoderAlloc(coder,size)	\
