@@ -25,6 +25,11 @@
 #include <signal.h>
 #ifdef HAVE_LIBGC
 #include <private/gc_priv.h>	/* For SIG_SUSPEND */
+#if defined(SIG_SUSPEND) && defined(GC_DARWIN_THREADS)
+/* SIG_SUSPEND is unused by LIBGC 6 for GC_DARWIN_THREADS and the definition */
+/* is wrong (SIGRTMIN + x). SIGRTMIN is not defined on MAC OSX */
+#undef SIG_SUSPEND
+#endif
 #endif
 
 #ifdef	__cplusplus
